#include "apconfigui.h"

#include <qapplication.h>
#include <qvariant.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qspinbox.h>
#include <qmime.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <asm/errno.h>
#include "authsecudlg.h"
#include "qhexvalidator.h"

#include "rt_tool.h"
#include "configapi.h"

// 2.4 Ghz channel plan
static UCHAR Ra24Ghz_FCC[] = {1,2,3,4,5,6,7,8,9,10,11};
static UCHAR Ra24Ghz_IC[] = {1,2,3,4,5,6,7,8,9,10,11};
static UCHAR Ra24Ghz_ESTI[]= {1,2,3,4,5,6,7,8,9,10,11,12,13};
static UCHAR Ra24Ghz_SPAIN[] = {10,11};
static UCHAR Ra24Ghz_FRANCE[] = {10,11,12,13};
static UCHAR Ra24Ghz_MKK[] = {14};
static UCHAR Ra24Ghz_MKK1[] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14};
static UCHAR Ra24Ghz_ISRAEL[] = {3,4,5,6,7,8,9};
    
// 5 Ghz channel plan
static UCHAR Ra5Ghz_UNII[] = {36,40,44,48,52,56,60,64,  149,153,157,161};
static UCHAR Ra5Ghz_MMAC[] = {34,38,42,46};
static UCHAR Ra5Ghz_HyperLAN2[] = {36,40,44,48,52,56,60,64,  100,104,108,112,116,120,124,128,132,136,140};

RT_802_11_PHY_MODE G_enumWirelessMode = PHY_11BG_MIXED; // Default use BG_Mixed
extern bool G_bSupportAMode;

PAIR_CHANNEL_FREQ_ENTRY ChannelFreqTable[] = 
{
    // channel Frequency
    {1,         2412000},
    {2,         2417000},
    {3,         2422000},
    {4,         2427000},
    {5,         2432000},
    {6,         2437000},
    {7,         2442000},
    {8,         2447000},
    {9,         2452000},
    {10,        2457000},
    {11,        2462000},
    {12,        2467000},
    {13,        2472000},
    {14,        2484000},
    {34,        5170000},
    {36,        5180000},
    {38,        5190000},
    {40,        5200000},
    {42,        5210000},
    {44,        5220000},
    {46,        5230000},
    {48,        5240000},
    {52,        5260000},
    {56,        5280000},
    {60,        5300000},
    {64,        5320000},
    {100,       5500000},
    {104,       5520000},
    {108,       5540000},
    {112,       5560000},
    {116,       5580000},
    {120,       5600000},
    {124,       5620000},
    {128,       5640000},
    {132,       5660000},
    {136,       5680000},
    {140,       5700000},
    {149,       5745000},
    {153,       5765000},
    {157,       5785000},
    {161,       5805000},
};

int G_nChanFreqCount = sizeof (ChannelFreqTable) / sizeof(PAIR_CHANNEL_FREQ_ENTRY);

extern int errno;
extern UCHAR CountryRegion;
extern bool G_bWpaCapable;
extern NDIS_802_11_ENCRYPTION_STATUS G_encryCapa;

static QPixmap uic_load_pixmap_ApConfigForm( const QString &name )
{
    const QMimeSource *m = QMimeSourceFactory::defaultFactory()->data( name );
    if ( !m )
        return QPixmap();
    QPixmap pix;
    QImageDrag::decode( m, pix );
    return pix;
}
/* 
 *  Constructs a ApConfig which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ApConfigForm::ApConfigForm( int socket_id, const char *device_name, QWidget *parent, const char *name)
    : QWidget( parent, name )
{
    QString                 qstr;
    char                    strtmp[812];
    int                     number;

    if(getkeystring("Default", "CountryRegion",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        CountryRegion = atoi(strtmp);

        if ((CountryRegion < REGSTR_COUNTRYREGION_MINIMUM) || (CountryRegion > REGSTR_COUNTRYREGION_MAXIMUM))
            CountryRegion = 0;
    }

    if ( !name )
        setName( "ApConfig" );
    resize( 600, 480 ); 
    setMinimumSize( 600, 480 ); 
    setMaximumSize( 600, 480 ); 
    setIcon( uic_load_pixmap_ApConfigForm( "apconfig2500.xpm" ) );
    setCaption( trUtf8( "Configuration Utility" ) );
    qApp->setStyle("Windows");

    TabWidgetApConfigForm = new QTabWidget( this, "TabWidgetApConfigForm" );
    TabWidgetApConfigForm->setGeometry( QRect( 10, 10, 580, 460 ) ); 
    setPaletteBackgroundColor( QColor( 222, 222,  222 ) );
    TabWidgetApConfigForm->setPaletteBackgroundColor( QColor( 222, 222,  222 ) );
//==========================================================================================
//ConfigPage
    tabConfigPage = new QWidget( TabWidgetApConfigForm, "ConfigPage" );

    TextLabelMode = new QLabel( tabConfigPage, "TextLabelMode" );
    TextLabelMode->setGeometry( QRect( 8, 28, 87, 20 ) ); 
    TextLabelMode->setText( trUtf8( "Wireless Mode" ) );

    ComboBoxMode = new QComboBox( FALSE, tabConfigPage, "ComboBoxMode" );
    ComboBoxMode->insertItem( trUtf8( "802.11 B/G mixed" ) );
    ComboBoxMode->insertItem( trUtf8( "802.11 B only" ) );
    ComboBoxMode->insertItem( trUtf8( "802.11 G only" ) );
    if (G_bSupportAMode)
    {
        ComboBoxMode->insertItem( trUtf8( "802.11 A only" ) );
        ComboBoxMode->insertItem( trUtf8( "802.11 A/B/G mixed" ) );
    }
    ComboBoxMode->setCurrentItem( 0 );
    ComboBoxMode->setGeometry( QRect( 110, 26, 130, 20 ) ); 

    if(getkeystring("Default", "WirelessMode",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if (G_bSupportAMode)
        {
            if((number>=0) && (number<=4))
                ComboBoxMode->setCurrentItem(number);
        }
        else
        {
            if((number>=0) && (number<=2))
                ComboBoxMode->setCurrentItem(number);
        }
    }

    TextLabel3 = new QLabel( tabConfigPage, "TextLabel3" );
    TextLabel3->setGeometry( QRect( 250, 27, 50, 20 ) ); 
    TextLabel3->setText( trUtf8( "TX Rate" ) );

    ComboBoxTxRate = new QComboBox( FALSE, tabConfigPage, "ComboBoxTxRate" );
    ComboBoxTxRate->insertItem( trUtf8( "Auto" ) );
    ComboBoxTxRate->insertItem( trUtf8( "1 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "2 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "5.5 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "11 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "6 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "9 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "12 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "18 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "24 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "36 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "48 Mbps" ) );
    ComboBoxTxRate->insertItem( trUtf8( "54 Mbps" ) );
    ComboBoxTxRate->setGeometry( QRect( 310, 26, 80, 20 ) ); 

    TextLabel4 = new QLabel( tabConfigPage, "TextLabel4" );
    TextLabel4->setGeometry( QRect( 400, 26, 60, 20 ) ); 
    TextLabel4->setText( trUtf8( "Channel" ) );

    GetCountryChannel();
    ComboBoxChannel = new QComboBox( FALSE, tabConfigPage, "ComboBoxChannel" );
    ComboBoxChannel->setGeometry( QRect( 470, 25, 70, 20 ) ); 
    {
        char channelString[3];
        int i;

        switch (ComboBoxMode->currentItem())
        {
            case PHY_11ABG_MIXED: // A/B/G Mixed
                for (i = 0; i < m_n24GhzIndex; i++)
                {
                    sprintf(channelString, "%d", m_chAryChan24Ghz[i]);
                    ComboBoxChannel->insertItem(channelString);
                }
                break;
            case PHY_11A: // A Only
                for (i = 0; i < m_n52GhzIndex; i++)
                {
                    sprintf(channelString, "%d", m_chAryChan52Ghz[i]);
                    ComboBoxChannel->insertItem(channelString);
                }
                ComboBoxChannel->setCurrentItem(0);
                break;
            case PHY_11B: // B only
            case PHY_11G: // G Only
            case PHY_11BG_MIXED: // B/G Mixed
            default:
                for (i = 0; i < m_n24GhzIndex; i++)
                {
                    sprintf(channelString, "%d", m_chAryChan24Ghz[i]);
                    ComboBoxChannel->insertItem(channelString);
                }
                ComboBoxChannel->setCurrentItem(0);
                break;
        }        
    }
    TextLabelSsid = new QLabel( tabConfigPage, "TextLabelSsid" );
    TextLabelSsid->setGeometry( QRect( 10, 65, 40, 20 ) ); 
    TextLabelSsid->setText( trUtf8( "SSID" ) );

    LineEditSsid = new QLineEdit( tabConfigPage, "LineEditSsid" );
    LineEditSsid->setMaxLength(32);
    LineEditSsid->setGeometry( QRect( 78, 65, 170, 20 ) ); 

    PushButtonUseMacAddress = new QPushButton( tabConfigPage, "PushButtonUseMacAddress" );
    PushButtonUseMacAddress->setGeometry( QRect( 268, 65, 130, 20 ) ); 
    PushButtonUseMacAddress->setText( trUtf8( "&USE MAC ADDRESS" ) );

    PushButtonSecurity = new QPushButton( tabConfigPage, "PushButtonSecurity" );
    PushButtonSecurity->setGeometry( QRect( 408, 65, 130, 20 ) ); 
    PushButtonSecurity->setText( trUtf8( "&SECURITY" ) );

    Line1 = new QFrame( tabConfigPage, "Line1" );
    Line1->setGeometry( QRect( 10, 93, 550, 20 ) ); 
    Line1->setFrameShape( QFrame::HLine );
    Line1->setFrameShadow( QFrame::Sunken );
    Line1->setFrameShape( QFrame::HLine );

    TextLabel5 = new QLabel( tabConfigPage, "TextLabel5" );
    TextLabel5->setGeometry( QRect( 8, 136, 110, 20 ) ); 
    TextLabel5->setText( trUtf8( "11 B/G Protection" ) );

    ComboBoxBGProtection = new QComboBox( FALSE, tabConfigPage, "ComboBoxBGProtection" );
    ComboBoxBGProtection->insertItem( trUtf8( "AUTO" ) );
    ComboBoxBGProtection->insertItem( trUtf8( "ON" ) );
    ComboBoxBGProtection->insertItem( trUtf8( "OFF" ) );
    ComboBoxBGProtection->setGeometry( QRect( 140, 136, 120, 20 ) ); 

    TextLabel8 = new QLabel( tabConfigPage, "TextLabel8" );
    TextLabel8->setGeometry( QRect( 8, 176, 100, 20 ) ); 
    TextLabel8->setText( trUtf8( "TX Preamble" ) );

    ComboBoxTXPreamble = new QComboBox( FALSE, tabConfigPage, "ComboBoxTXPreamble" );
    ComboBoxTXPreamble->insertItem( trUtf8( "Long" ) );
    ComboBoxTXPreamble->insertItem( trUtf8( "Short" ) );
    ComboBoxTXPreamble->setGeometry( QRect( 140, 176, 120, 20 ) ); 

    TextLabel9 = new QLabel( tabConfigPage, "TextLabel9" );
    TextLabel9->setGeometry( QRect( 8, 216, 100, 20 ) ); 
    TextLabel9->setText( trUtf8( "Beacon Period (ms)" ) );

    SpinBoxBeaconPeriod = new QSpinBox( tabConfigPage, "SpinBoxBeaconPeriod" );
    SpinBoxBeaconPeriod->setGeometry( QRect( 140, 216, 120, 20 ) ); 
    SpinBoxBeaconPeriod->setMinValue(50);
    SpinBoxBeaconPeriod->setMaxValue(1024);
    SpinBoxBeaconPeriod->setValue(100);

    TextLabel10 = new QLabel( tabConfigPage, "TextLabel10" );
    TextLabel10->setGeometry( QRect( 8, 256, 100, 20 ) ); 
    TextLabel10->setText( trUtf8( "RTS Threshold" ) );

    SpinBoxRtsThreshold = new QSpinBox( tabConfigPage, "SpinBoxRtsThreshold" );
    SpinBoxRtsThreshold->setGeometry( QRect( 140, 256, 120, 20 ) ); 
    SpinBoxRtsThreshold->setMinValue( 1 );
    SpinBoxRtsThreshold->setMaxValue( 2347 );
    SpinBoxRtsThreshold->setValue( 2347 );
    SpinBoxRtsThreshold->setEnabled( FALSE );

    TextLabel11 = new QLabel( tabConfigPage, "TextLabel11" );
    TextLabel11->setGeometry( QRect( 10, 296, 120, 20 ) ); 
    TextLabel11->setText( trUtf8( "Fragment Threshold" ) );

    SpinBoxFragThreshold = new QSpinBox( tabConfigPage, "SpinBoxFragThreshold" );
    SpinBoxFragThreshold->setGeometry( QRect( 140, 296, 120, 20 ) ); 
    SpinBoxFragThreshold->setMinValue( 256 );
    SpinBoxFragThreshold->setMaxValue( 2346 );
    SpinBoxFragThreshold->setValue( 2346 );
    SpinBoxFragThreshold->setEnabled( FALSE );

    CheckBoxTXBurst = new QCheckBox( tabConfigPage, "CheckBoxTXBurst" );
    CheckBoxTXBurst->setGeometry( QRect( 270, 136, 250, 20 ) ); 
    CheckBoxTXBurst->setText( trUtf8( "&Enable TX Burst" ) );

    CheckBoxTurboRate = new QCheckBox( tabConfigPage, "CheckBoxTurboRate" );
    CheckBoxTurboRate->setGeometry( QRect( 270, 176, 290, 20 ) ); 
    CheckBoxTurboRate->setText( trUtf8( "Use Turbo rate (72/100Mbps) when applicable" ) );
    CheckBoxTurboRate->setEnabled( FALSE );

    CheckBoxNoForwarding = new QCheckBox( tabConfigPage, "CheckBoxNoForwarding" );
    CheckBoxNoForwarding->setGeometry( QRect( 270, 216, 270, 20 ) ); 
    CheckBoxNoForwarding->setText( trUtf8( "No forwarding among wireless clients" ) );

    CheckBoxHideSSID = new QCheckBox( tabConfigPage, "CheckBoxHideSSID" );
    CheckBoxHideSSID->setGeometry( QRect( 270, 256, 170, 20 ) ); 
    CheckBoxHideSSID->setText( trUtf8( "Hide SSID" ) );

    CheckBoxShortSlot = new QCheckBox( tabConfigPage, "CheckBoxShortSlot" );
    CheckBoxShortSlot->setGeometry( QRect( 270, 296, 240, 20 ) ); 
    CheckBoxShortSlot->setText( trUtf8( "Use Short Slot Time when applicable" ) );

    CheckBoxAutoSelChannel = new QCheckBox( tabConfigPage, "CheckBoxAutoSelChannel" );
    CheckBoxAutoSelChannel->setGeometry( QRect( 270, 336, 240, 20 ) ); 
    CheckBoxAutoSelChannel->setText( trUtf8( "Auto Channel Selection at next boot" ) );

    PushButtonDefault = new QPushButton( tabConfigPage, "PushButtonDefault" );
    PushButtonDefault->setGeometry( QRect( 298, 376, 110, 20 ) ); 
    PushButtonDefault->setText( trUtf8( "DEFAULT" ) );

    PushButtonApply = new QPushButton( tabConfigPage, "PushButtonApply" );
    PushButtonApply->setGeometry( QRect( 418, 376, 120, 20 ) ); 
    PushButtonApply->setText( trUtf8( "APPLY" ) );

    TabWidgetApConfigForm->insertTab( tabConfigPage, trUtf8( "&Config" ) );

//==========================================================================================
//AccessControlPage
    tabAccessControlPage = new QWidget( TabWidgetApConfigForm, "tabAccessControlPage" );

    TextLabel12 = new QLabel( tabAccessControlPage, "TextLabel12" );
    TextLabel12->setGeometry( QRect( 10, 30, 230, 20 ) ); 
    TextLabel12->setText( trUtf8( "Wireless Client Access Control Policy" ) );

    ComboBoxAccessPolicy = new QComboBox( FALSE, tabAccessControlPage, "ComboBoxAccessPolicy" );
    ComboBoxAccessPolicy->insertItem( trUtf8( "Disable" ) );
    ComboBoxAccessPolicy->insertItem( trUtf8( "Allow All" ) );
    ComboBoxAccessPolicy->insertItem( trUtf8( "Reject All" ) );
    ComboBoxAccessPolicy->setGeometry( QRect( 280, 30, 240, 20 ) ); 

    TextLabel13 = new QLabel( tabAccessControlPage, "TextLabel13" );
    TextLabel13->setGeometry( QRect( 10, 70, 90, 20 ) ); 
    TextLabel13->setText( trUtf8( "MAC Address" ) );

    hexValidator = new QHexValidator(this);

    LineEditMacAddress = new QLineEdit( tabAccessControlPage, "LineEditMacAddress" );
    LineEditMacAddress->setGeometry( QRect( 110, 70, 160, 20 ) ); 
    LineEditMacAddress->setValidator(hexValidator);
    LineEditMacAddress->setMaxLength(12);
    LineEditMacAddress->setEnabled( FALSE );

    AccessPushButtonAdd = new QPushButton( tabAccessControlPage, "AccessPushButtonAdd" );
    AccessPushButtonAdd->setGeometry( QRect( 110, 140, 120, 20 ) ); 
    AccessPushButtonAdd->setText( trUtf8( "Add" ) );
    AccessPushButtonAdd->setEnabled( FALSE );

    AccessPushButtonDelete = new QPushButton( tabAccessControlPage, "AccessPushButtonDelete" );
    AccessPushButtonDelete->setGeometry( QRect( 110, 180, 120, 20 ) ); 
    AccessPushButtonDelete->setText( trUtf8( "Delete" ) );
    AccessPushButtonDelete->setEnabled( FALSE );

    AccessPushButtonRemoveAll = new QPushButton( tabAccessControlPage, "AccessPushButtonRemoveAll" );
    AccessPushButtonRemoveAll->setGeometry( QRect( 110, 220, 120, 20 ) ); 
    AccessPushButtonRemoveAll->setText( trUtf8( "Remove all" ) );
    AccessPushButtonRemoveAll->setEnabled( FALSE );

    TextLabel14 = new QLabel( tabAccessControlPage, "TextLabel14" );
    TextLabel14->setGeometry( QRect( 280, 70, 210, 20 ) ); 
    TextLabel14->setText( trUtf8( "Access Control List" ) );

    ListViewAccessControl = new QListView( tabAccessControlPage, "ListViewAccessControl" );        
    ListViewAccessControl->addColumn( trUtf8( "MAC Address" ) );
    ListViewAccessControl->setSelectionMode( QListView::Single );
    ListViewAccessControl->setAllColumnsShowFocus( TRUE );
    ListViewAccessControl->setEnabled( FALSE );
    ListViewAccessControl->setGeometry( QRect( 280, 100, 280, 280 ) ); 

    AccessPushButtonApplyNow = new QPushButton( tabAccessControlPage, "AccessPushButtonApplyNow" );
    AccessPushButtonApplyNow->setGeometry( QRect( 440, 390, 120, 20 ) ); 
    AccessPushButtonApplyNow->setText( trUtf8( "A&pply" ) );

    TabWidgetApConfigForm->insertTab( tabAccessControlPage, trUtf8( "&Access Control" ) );

//==========================================================================================
//WdsPage
    tabWdsPage = new QWidget( TabWidgetApConfigForm, "tabWdsPage" );

    ButtonGroupWds = new QButtonGroup( tabWdsPage, "ButtonGroupWds" );
    ButtonGroupWds->setGeometry( QRect( 20, 10, 540, 80 ) ); 
    ButtonGroupWds->setTitle( trUtf8( "WDS" ) );

    TextLabelWdsPolicy = new QLabel( ButtonGroupWds, "TextLabelWdsPolicy" );
    TextLabelWdsPolicy->setGeometry( QRect( 20, 30, 110, 30 ) ); 
    TextLabelWdsPolicy->setText( trUtf8( "WDS control policy" ) );

    RadioButtonWdsEnable = new QRadioButton( ButtonGroupWds, "RadioButtonWdsEnable" );
    RadioButtonWdsEnable->setGeometry( QRect( 210, 30, 80, 30 ) ); 
    RadioButtonWdsEnable->setText( trUtf8( "Enable" ) );
    ButtonGroupWds->insert( RadioButtonWdsEnable, 0 );

    RadioButtonWdsDisable = new QRadioButton( ButtonGroupWds, "RadioButtonWdsDisable" );
    RadioButtonWdsDisable->setGeometry( QRect( 330, 30, 100, 30 ) ); 
    RadioButtonWdsDisable->setText( trUtf8( "Disable" ) );
    RadioButtonWdsDisable->setChecked( TRUE );
    ButtonGroupWds->insert( RadioButtonWdsDisable, 1 );

    TextLabelWdsMacAddress = new QLabel( tabWdsPage, "TextLabelWdsMacAddress" );
    TextLabelWdsMacAddress->setGeometry( QRect( 20, 110, 80, 20 ) ); 
    TextLabelWdsMacAddress->setText( trUtf8( "MAC Address" ) );

    LineEditWdsMacAddress = new QLineEdit( tabWdsPage, "LineEditWdsMacAddress" );
    LineEditWdsMacAddress->setEnabled( FALSE );
    LineEditWdsMacAddress->setValidator(hexValidator);
    LineEditWdsMacAddress->setMaxLength(12);
    LineEditWdsMacAddress->setGeometry( QRect( 110, 110, 160, 20 ) ); 

    WdsPushButtonAdd = new QPushButton( tabWdsPage, "WdsPushButtonAdd" );
    WdsPushButtonAdd->setEnabled( FALSE );
    WdsPushButtonAdd->setGeometry( QRect( 170, 170, 100, 30 ) );
    WdsPushButtonAdd->setText( trUtf8( "ADD" ) );

    WdsPushButtonDelete = new QPushButton( tabWdsPage, "WdsPushButtonDelete" );
    WdsPushButtonDelete->setEnabled( FALSE );
    WdsPushButtonDelete->setGeometry( QRect( 170, 210, 100, 30 ) ); 
    WdsPushButtonDelete->setText( trUtf8( "DELETE" ) );

    WdsPushButtonRemoveAll = new QPushButton( tabWdsPage, "WdsPushButtonRemoveAll" );
    WdsPushButtonRemoveAll->setEnabled( FALSE );
    WdsPushButtonRemoveAll->setGeometry( QRect( 170, 250, 100, 30 ) ); 
    WdsPushButtonRemoveAll->setText( trUtf8( "REMOVE ALL" ) );

    TextLabelWdsState = new QLabel( tabWdsPage, "TextLabelWdsState" );
    TextLabelWdsState->setGeometry( QRect( 18, 340, 280, 90 ) ); 
    TextLabelWdsState->setText( trUtf8( "Each WDS must be config as the same channel and \n"
                                    "wep status. Furthermore there can not be a loop between\n"
                                    "them" ) );

    TextLabelDumbApList = new QLabel( tabWdsPage, "TextLabelDumbApList" );
    TextLabelDumbApList->setGeometry( QRect( 310, 110, 110, 20 ) ); 
    TextLabelDumbApList->setText( trUtf8( "WDS List-" ) );

    ListViewWds = new QListView( tabWdsPage, "ListViewWds" );
    ListViewWds->addColumn( trUtf8( "Mac Address" ) );
    ListViewWds->setSelectionMode( QListView::Single );
    ListViewWds->setAllColumnsShowFocus( TRUE );
    ListViewWds->setEnabled( FALSE );
    ListViewWds->setGeometry( QRect( 310, 140, 250, 220 ) ); 

    WdsPushButtonApply = new QPushButton( tabWdsPage, "WdsPushButtonApply" );
    WdsPushButtonApply->setGeometry( QRect( 460, 370, 100, 30 ) ); 
    WdsPushButtonApply->setText( trUtf8( "APPLY" ) );

    TabWidgetApConfigForm->insertTab( tabWdsPage, trUtf8( "WDS" ) );

//==========================================================================================
//MacTablePage
    tabMacTablePage = new QWidget( TabWidgetApConfigForm, "tabMacTablePage" );

    ListViewMacTable = new QListView( tabMacTablePage, "ListViewMacTable" );
    ListViewMacTable->addColumn( trUtf8( "MAC Address" ) );
    ListViewMacTable->addColumn( trUtf8( "AID" ) );
    ListViewMacTable->addColumn( trUtf8( "Power Saving" ) );
    ListViewMacTable->setGeometry( QRect( 11, 10, 550, 410 ) ); 
    TabWidgetApConfigForm->insertTab( tabMacTablePage, trUtf8( "&Mac Table" ) );

//==========================================================================================
//EventLogPage
    tabEventLogPage = new QWidget( TabWidgetApConfigForm, "tabEventLogPage" );

    ListViewEventLog = new QListView( tabEventLogPage, "ListViewEventLog" );
    ListViewEventLog->addColumn( trUtf8( "Event Time" ) );
    ListViewEventLog->addColumn( trUtf8( "Message" ) );
    ListViewEventLog->setSelectionMode( QListView::Single );
    ListViewEventLog->setGeometry( QRect( 12, 10, 550, 380 ) ); 
    ListViewEventLog->setAllColumnsShowFocus( TRUE );

    EventPushButtonClear = new QPushButton( tabEventLogPage, "EventPushButtonClear" );
    EventPushButtonClear->setGeometry( QRect( 465, 396, 100, 30 ) ); 
    EventPushButtonClear->setText( trUtf8( "&Clear" ) );

    TabWidgetApConfigForm->insertTab( tabEventLogPage, trUtf8( "E&vent Log" ) );

//==========================================================================================
//StatisticsPage
    tabStatisticsPage = new QWidget( TabWidgetApConfigForm, "tabStatisticsPage" );

    GroupBox1 = new QGroupBox( tabStatisticsPage, "GroupBox1" );
    GroupBox1->setGeometry( QRect( 19, 16, 540, 180 ) ); 
    GroupBox1->setTitle( trUtf8( "Transmit Statistics" ) );

    TextLabel38 = new QLabel( GroupBox1, "TextLabel38" );
    TextLabel38->setGeometry( QRect( 10, 20, 260, 20 ) ); 
    TextLabel38->setText( trUtf8( "Frames Transmitted Successfully" ) );

    TextLabelEqlsign1 = new QLabel( GroupBox1, "TextLabelEqlsign1" );
    TextLabelEqlsign1->setGeometry( QRect( 290, 20, 20, 20 ) ); 
    TextLabelEqlsign1->setText( trUtf8( "=" ) );

    TranSucFrameLable = new QLabel( GroupBox1, "TranSucFrameLable" );
    TranSucFrameLable->setGeometry( QRect( 340, 20, 170, 20 ) ); 
    TranSucFrameLable->setText( trUtf8( "0" ) );
    TranSucFrameLable->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel39 = new QLabel( GroupBox1, "TextLabel39" );
    TextLabel39->setGeometry( QRect( 10, 45, 280, 20 ) ); 
    TextLabel39->setText( trUtf8( "Frames Transmitted Successfully Without Retry" ) );

    TextLabelEqlsign2 = new QLabel( GroupBox1, "TextLabelEqlsign2" );
    TextLabelEqlsign2->setGeometry( QRect( 290, 45, 20, 20 ) ); 
    TextLabelEqlsign2->setText( trUtf8( "=" ) );

    TranSucWithoutRetryLabel = new QLabel( GroupBox1, "TranSucWithoutRetryLabel" );
    TranSucWithoutRetryLabel->setGeometry( QRect( 340, 45, 170, 20 ) ); 
    TranSucWithoutRetryLabel->setText( trUtf8( "0" ) );
    TranSucWithoutRetryLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel40 = new QLabel( GroupBox1, "TextLabel40" );
    TextLabel40->setGeometry( QRect( 10, 70, 300, 20 ) ); 
    TextLabel40->setText( trUtf8( "Frames Transmitted Successfully After Retry(s)" ) );

    TextLabelEqlsign3 = new QLabel( GroupBox1, "TextLabelEqlsign3" );
    TextLabelEqlsign3->setGeometry( QRect( 290, 70, 20, 20 ) ); 
    TextLabelEqlsign3->setText( trUtf8( "=" ) );

    TranSucAftRetryLabel = new QLabel( GroupBox1, "TranSucAftRetryLabel" );
    TranSucAftRetryLabel->setGeometry( QRect( 340, 70, 170, 20 ) ); 
    TranSucAftRetryLabel->setText( trUtf8( "0" ) );
    TranSucAftRetryLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel41 = new QLabel( GroupBox1, "TextLabel41" );
    TextLabel41->setGeometry( QRect( 10, 95, 320, 20 ) ); 
    TextLabel41->setText( trUtf8( "Frames Fail To Receive ACK After All Retries" ) );

    TextLabelEqlsign4 = new QLabel( GroupBox1, "TextLabelEqlsign4" );
    TextLabelEqlsign4->setGeometry( QRect( 290, 95, 20, 20 ) ); 
    TextLabelEqlsign4->setText( trUtf8( "=" ) );

    TranFailAckLabel = new QLabel( GroupBox1, "TranFailAckLabel" );
    TranFailAckLabel->setGeometry( QRect( 340, 95, 170, 20 ) ); 
    TranFailAckLabel->setText( trUtf8( "0" ) );
    TranFailAckLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel42 = new QLabel( GroupBox1, "TextLabel42" );
    TextLabel42->setGeometry( QRect( 10, 120, 340, 20 ) ); 
    TextLabel42->setText( trUtf8( "RTS Frames Successfully Receive CTS" ) );

    TextLabelEqlsign5 = new QLabel( GroupBox1, "TextLabelEqlsign5" );
    TextLabelEqlsign5->setGeometry( QRect( 290, 120, 20, 20 ) ); 
    TextLabelEqlsign5->setText( trUtf8( "=" ) );

    TranSucRTSLabel = new QLabel( GroupBox1, "TranSucRTSLabel" );
    TranSucRTSLabel->setGeometry( QRect( 340, 120, 170, 20 ) ); 
    TranSucRTSLabel->setText( trUtf8( "0" ) );
    TranSucRTSLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel43 = new QLabel( GroupBox1, "TextLabel43" );
    TextLabel43->setGeometry( QRect( 11, 145, 280, 20 ) ); 
    TextLabel43->setText( trUtf8( "RTS Frames Fail To Receive CTS" ) );

    TextLabelEqlsign6 = new QLabel( GroupBox1, "TextLabelEqlsign6" );
    TextLabelEqlsign6->setGeometry( QRect( 290, 145, 20, 20 ) ); 
    TextLabelEqlsign6->setText( trUtf8( "=" ) );

    TranFailRTSLabel = new QLabel( GroupBox1, "TranFailRTSLabel" );
    TranFailRTSLabel->setGeometry( QRect( 340, 145, 170, 20 ) ); 
    TranFailRTSLabel->setText( trUtf8( "0" ) );
    TranFailRTSLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    GroupBox2 = new QGroupBox( tabStatisticsPage, "GroupBox2" );
    GroupBox2->setGeometry( QRect( 20, 210, 541, 130 ) ); 
    GroupBox2->setTitle( trUtf8( "Receive Statistics" ) );

    TextLabel28 = new QLabel( GroupBox2, "TextLabel28" );
    TextLabel28->setGeometry( QRect( 10, 20, 180, 20 ) ); 
    TextLabel28->setText( trUtf8( "Frames Received Successfully" ) );

    TextLabelEqlsign7 = new QLabel( GroupBox2, "TextLabelEqlsign7" );
    TextLabelEqlsign7->setGeometry( QRect( 290, 20, 20, 20 ) ); 
    TextLabelEqlsign7->setText( trUtf8( "=" ) );

    RcvSucFrameLabel = new QLabel( GroupBox2, "RcvSucFrameLabel" );
    RcvSucFrameLabel->setGeometry( QRect( 340, 20, 170, 20 ) ); 
    RcvSucFrameLabel->setText( trUtf8( "0" ) );
    RcvSucFrameLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel29 = new QLabel( GroupBox2, "TextLabel29" );
    TextLabel29->setGeometry( QRect( 10, 45, 200, 20 ) ); 
    TextLabel29->setText( trUtf8( "Frames Received With CRC Error" ) );

    TextLabelEqlsign8 = new QLabel( GroupBox2, "TextLabelEqlsign8" );
    TextLabelEqlsign8->setGeometry( QRect( 290, 45, 20, 20 ) ); 
    TextLabelEqlsign8->setText( trUtf8( "=" ) );

    RcvCRCErrLabel = new QLabel( GroupBox2, "RcvCRCErrLabel" );
    RcvCRCErrLabel->setGeometry( QRect( 340, 45, 170, 20 ) ); 
    RcvCRCErrLabel->setText( trUtf8( "0" ) );
    RcvCRCErrLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel30 = new QLabel( GroupBox2, "TextLabel30" );
    TextLabel30->setGeometry( QRect( 10, 70, 250, 20 ) ); 
    TextLabel30->setText( trUtf8( "Frames Dropped Due To Out-of-Resource" ) );

    TextLabelEqlsign9 = new QLabel( GroupBox2, "TextLabelEqlsign9" );
    TextLabelEqlsign9->setGeometry( QRect( 290, 70, 20, 20 ) ); 
    TextLabelEqlsign9->setText( trUtf8( "=" ) );

    RcvDropFrameLabel = new QLabel( GroupBox2, "RcvDropFrameLabel" );
    RcvDropFrameLabel->setGeometry( QRect( 340, 70, 170, 20 ) ); 
    RcvDropFrameLabel->setText( trUtf8( "0" ) );
    RcvDropFrameLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    TextLabel31 = new QLabel( GroupBox2, "TextLabel31" );
    TextLabel31->setGeometry( QRect( 10, 95, 230, 20 ) ); 
    TextLabel31->setText( trUtf8( "Duplicate Frames Received" ) );

    TextLabelEqlsign10 = new QLabel( GroupBox2, "TextLabelEqlsign10" );
    TextLabelEqlsign10->setGeometry( QRect( 290, 95, 20, 20 ) ); 
    TextLabelEqlsign10->setText( trUtf8( "=" ) );

    RcvDupFrameLabel = new QLabel( GroupBox2, "RcvDupFrameLabel" );
    RcvDupFrameLabel->setGeometry( QRect( 340, 95, 170, 20 ) ); 
    RcvDupFrameLabel->setText( trUtf8( "0" ) );
    RcvDupFrameLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    StatisticsPushButtonReset = new QPushButton( tabStatisticsPage, "StatisticsPushButtonReset" );
    StatisticsPushButtonReset->setGeometry( QRect( 418, 356, 130, 30 ) ); 
    StatisticsPushButtonReset->setText( trUtf8( "Reset Counters" ) );

    TabWidgetApConfigForm->insertTab( tabStatisticsPage, trUtf8( "&Statistics" ) );

//==========================================================================================
//AboutPage

    tabAboutPage = new QWidget( TabWidgetApConfigForm, "tabAboutPage" );

    TextLabel1_2 = new QLabel( tabAboutPage, "TextLabel1_2" );
    TextLabel1_2->setGeometry( QRect( 28, 36, 190, 90 ) ); 
    TextLabel1_2->setText( trUtf8( "" ) );
    TextLabel1_2->setPixmap( uic_load_pixmap_ApConfigForm("rtlogo.png" ) );

    GroupBox3 = new QGroupBox( tabAboutPage, "GroupBox3" );
    GroupBox3->setGeometry( QRect( 30, 140, 520, 60 ) ); 
    GroupBox3->setTitle( trUtf8( "Configuration Utility" ) );

    TextLabel33 = new QLabel( GroupBox3, "TextLabel33" );
    TextLabel33->setGeometry( QRect( 20, 20, 70, 20 ) ); 
    TextLabel33->setText( trUtf8( "Version:" ) );

    UtilityVersionTextLabel = new QLabel( GroupBox3, "UtilityVersionTextLabel" );
    UtilityVersionTextLabel->setGeometry( QRect( 100, 20, 100, 20 ) ); 
    UtilityVersionTextLabel->setText( trUtf8( CONFIGURATION_UI_VERSION ) );

    TextLabel34 = new QLabel( GroupBox3, "TextLabel34" );
    TextLabel34->setGeometry( QRect( 280, 20, 50, 20 ) ); 
    TextLabel34->setText( trUtf8( "Date:" ) );

    UtilityDateTextLabel = new QLabel( GroupBox3, "UtilityDateTextLabel" );
    UtilityDateTextLabel->setGeometry( QRect( 360, 20, 100, 20 ) ); 
    UtilityDateTextLabel->setText( trUtf8( CONFIGURATION_UI_DATE ) );

    GroupBox4 = new QGroupBox( tabAboutPage, "GroupBox4" );
    GroupBox4->setGeometry( QRect( 30, 220, 520, 60 ) ); 
    GroupBox4->setTitle( trUtf8( "NIC Driver" ) );

    TextLabel35 = new QLabel( GroupBox4, "TextLabel35" );
    TextLabel35->setGeometry( QRect( 20, 20, 60, 20 ) ); 
    TextLabel35->setText( trUtf8( "Version:" ) );

    NicVersionTextLabel = new QLabel( GroupBox4, "NicVersionTextLabel" );
    NicVersionTextLabel->setGeometry( QRect( 100, 20, 100, 20 ) ); 
    NicVersionTextLabel->setText( trUtf8( "" ) );

    TextLabel36 = new QLabel( GroupBox4, "TextLabel36" );
    TextLabel36->setGeometry( QRect( 280, 20, 40, 20 ) ); 
    TextLabel36->setText( trUtf8( "Date:" ) );

    NicDateTextLabel = new QLabel( GroupBox4, "NicDateTextLabel" );
    NicDateTextLabel->setGeometry( QRect( 360, 20, 100, 20 ) ); 
    NicDateTextLabel->setText( trUtf8( "" ) );

    GroupBox5 = new QGroupBox( tabAboutPage, "GroupBox5" );
    GroupBox5->setGeometry( QRect( 30, 300, 520, 60 ) ); 
    GroupBox5->setTitle( trUtf8( "Mac Address" ) );

    TextLabel37 = new QLabel( GroupBox5, "TextLabel37" );
    TextLabel37->setGeometry( QRect( 20, 20, 90, 20 ) ); 
    TextLabel37->setText( trUtf8( "Phy_Address:" ) );

    MacTextLabel = new QLabel( GroupBox5, "MacTextLabel" );
    MacTextLabel->setGeometry( QRect( 150, 20, 180, 20 ) ); 
    MacTextLabel->setText( trUtf8( "" ) );

    TabWidgetApConfigForm->insertTab( tabAboutPage, trUtf8( "A&bout" ) );

    // signals and slots connections
    connect( TabWidgetApConfigForm, SIGNAL( currentChanged(QWidget*) ), this, SLOT( OnPageChanged() ) );
    connect( PushButtonApply, SIGNAL( clicked() ), this, SLOT( ConfigPage_OnApplyNow() ) );
    connect( PushButtonSecurity, SIGNAL( clicked() ), this, SLOT( ConfigPage_OnSecurity() ) );
    connect( PushButtonUseMacAddress, SIGNAL( clicked() ), this, SLOT( ConfigPage_OnUseMacAddress() ) );
    connect( PushButtonDefault, SIGNAL( clicked() ), this, SLOT( ConfigPage_OnDefault() ) );
    connect( ComboBoxMode, SIGNAL( activated(int) ), this, SLOT( ConfigPage_ComboBoxMode_activated(int) ) );
    connect( AccessPushButtonApplyNow, SIGNAL( clicked() ), this, SLOT( AccessControlPage_OnApplyNow() ) );
    connect( AccessPushButtonAdd, SIGNAL( clicked() ), this, SLOT( AccessControlPage_OnAdd() ) );
    connect( AccessPushButtonDelete, SIGNAL( clicked() ), this, SLOT( AccessControlPage_OnDelete() ) );
    connect( AccessPushButtonRemoveAll, SIGNAL( clicked() ), this, SLOT( AccessControlPage_OnRemoveAll() ) );
    connect( ComboBoxAccessPolicy, SIGNAL( activated(int) ), this, SLOT( AccessControlPage_OnSelchangeAccessPolicy(int) ) );
    connect( ButtonGroupWds, SIGNAL( clicked(int) ), this, SLOT( WdsPage_ButtonGroupWds_Click(int) ) );
    connect( WdsPushButtonApply, SIGNAL( clicked() ), this, SLOT( WdsPage_OnApplyNow() ) );
    connect( WdsPushButtonAdd, SIGNAL( clicked() ), this, SLOT( WdsPage_OnAdd() ) );
    connect( WdsPushButtonDelete, SIGNAL( clicked() ), this, SLOT( WdsPage_OnDelete() ) );
    connect( WdsPushButtonRemoveAll, SIGNAL( clicked() ), this, SLOT( WdsPage_OnRemoveAll() ) );
    connect( EventPushButtonClear, SIGNAL( clicked() ), this, SLOT( EventLogPage_OnClear() ) );
    connect( StatisticsPushButtonReset, SIGNAL( clicked() ), this, SLOT( StatisticsPage_OnResetCounters() ) );

    m_nSocket_Id = socket_id;
    m_strDevice_Name = (char *)malloc(strlen(device_name)+1);
    if(m_strDevice_Name)
        strcpy(m_strDevice_Name, device_name);

    m_pdlgAuthSecu = new AuthSecuDlg(this, "Authentication && Security", TRUE);

    timerId_alive = startTimer(2000); //check device driver per 2 seconds.
    timerId_getMacTable = -1;
    timerId_getEventLog = -1;
    timerId_getStatistics = -1;
    m_n24GhzIndex = 0;
    m_n52GhzIndex = 0;
    G_nEventRead = 0;
    G_aryEventLog.clear();

    if(getkeystring("Default", "SSID",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        if(strlen(strtmp) > 32)
            strtmp[32]=0x00;
        LineEditSsid->setText( strtmp );
    }

    if(getkeystring("Default", "TxRate",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        switch(ComboBoxMode->currentItem())
        {
            case PHY_11BG_MIXED:
            case PHY_11G:
                if( (number >= 0) && (number <= 12))
                    ComboBoxTxRate->setCurrentItem(number);
                break;
            case PHY_11B:
                if( (number >= 0) && (number <= 4))
                    ComboBoxTxRate->setCurrentItem(number);
                break;
            case PHY_11A:
                if( (number >= 0) && (number <= 8))
                    ComboBoxTxRate->setCurrentItem(number);
                break;
        }
    }

    if(getkeystring("Default", "AutoChannelSelect",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if (number == 0)
        {
            CheckBoxAutoSelChannel->setChecked( FALSE );
            m_bAutoChanSel = FALSE;
        }
        else
        {
            CheckBoxAutoSelChannel->setChecked( TRUE );
            m_bAutoChanSel = TRUE;
        }
    }

    if (m_bAutoChanSel != TRUE)
    {
        if(getkeystring("Default", "Channel",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
        {
            QString text;
            int i;

            number = atoi(strtmp);
            qstr.sprintf("%d", number);
    
            for (i = 0; i < ComboBoxChannel->count(); i++)
            {
                text = ComboBoxChannel->text(i);
                if (text == qstr)
                {
                    ComboBoxChannel->setCurrentItem( i );
                    break;
                }
            }

            if (i >= ComboBoxChannel->count())
            {//not found
                ComboBoxChannel->setCurrentItem( 0 );
            }
        }
    }
    else
    {
        NDIS_802_11_CONFIGURATION Configuration;
        ULONG   channel = 1;
        QString text;
        int i;

        OidQueryInformation(OID_802_11_CONFIGURATION, m_nSocket_Id, m_strDevice_Name, &Configuration, sizeof(Configuration));
        for(i = 0; i < G_nChanFreqCount; i++)
        {
            if (Configuration.DSConfig == ChannelFreqTable[i].lFreq)
                channel = ChannelFreqTable[i].lChannel;
        }

        qstr.sprintf("%d", channel);

        for (i = 0; i < ComboBoxChannel->count(); i++)
        {
            text = ComboBoxChannel->text(i);
            if (text == qstr)
            {
                ComboBoxChannel->setCurrentItem( i );
                break;
            }
        }

        if (i >= ComboBoxChannel->count())
        {//not found
            ComboBoxChannel->setCurrentItem( 0 );
        }
    }

    if(getkeystring("Default", "BeaconPeriod",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);
        if( (number >= 1) && (number <= 1024) )
        {
            SpinBoxBeaconPeriod->setValue( number );
        }
    }

    if(getkeystring("Default", "BGProtection",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(ComboBoxMode->currentItem() == PHY_11B) //802.11 B only
        {
            ComboBoxBGProtection->setEnabled( FALSE );
            ComboBoxBGProtection->setCurrentItem( 0 );
        }
        else if (ComboBoxMode->currentItem() == PHY_11G) //802.11 G only
        {
            ComboBoxBGProtection->setEnabled( FALSE );
            if( (number >=0) && (number <= 2))
                ComboBoxBGProtection->setCurrentItem( number );
        }
        else
        {
            if( (number >=0) && (number <= 2))
                ComboBoxBGProtection->setCurrentItem( number );
        }
    }

    if(getkeystring("Default", "TxPreamble",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if( (number == 0) || (number == 1) )
            ComboBoxTXPreamble->setCurrentItem( number );
    }

    if(getkeystring("Default", "RTSThreshold",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if( (number >= 1) && (number <= 2347))
            SpinBoxRtsThreshold->setValue( number );
    }

    if(getkeystring("Default", "FragThreshold",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if( (number >= 256) && (number <= 2346))
            SpinBoxFragThreshold->setValue( number );
    }

    if(getkeystring("Default", "TxBurst",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(number == 0)
            CheckBoxTXBurst->setChecked( FALSE );
        else
            CheckBoxTXBurst->setChecked( TRUE );
    }

    if(getkeystring("Default", "TurboRate",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(number == 0)
            CheckBoxTurboRate->setChecked( FALSE );
        else
            CheckBoxTurboRate->setChecked( TRUE );
    }

    if(getkeystring("Default", "NoForwarding",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(number == 0)
            CheckBoxNoForwarding->setChecked( FALSE );
        else
            CheckBoxNoForwarding->setChecked( TRUE );
    }

    if(getkeystring("Default", "HideSSID",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(number == 0)
            CheckBoxHideSSID->setChecked( FALSE );
        else
            CheckBoxHideSSID->setChecked( TRUE );
    }

    if(getkeystring("Default", "ShortSlot",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if(number == 0)
            CheckBoxShortSlot->setChecked( FALSE );
        else
            CheckBoxShortSlot->setChecked( TRUE );
    }

    if(getkeystring("Default", "AccessControlList",  strtmp, 812, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        QListViewItem   *Item=NULL;
        QString         strAddress;
        unsigned char   macAddress[ETH_LENGTH_OF_ADDRESS];
        char            *macptr;
        int             i;

        for (macptr = strtok(strtmp,";"); macptr; macptr = strtok(NULL,";")) 
        {
            if(strlen(macptr) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
                continue; 

            for (i=0; i<ETH_LENGTH_OF_ADDRESS; i++)
            {
                AtoH(macptr, &macAddress[i], 2);
                macptr=macptr+3;
            }

            strAddress.sprintf("%02X-%02X-%02X-%02X-%02X-%02X", 
                macAddress[0], macAddress[1], macAddress[2], 
                macAddress[3], macAddress[4], macAddress[5]);

            Item = ListViewAccessControl->findItem(strAddress, 0);
            if(!Item)
            {//not found.
                Item = new QListViewItem(ListViewAccessControl, 0);
                Item->setText(0, strAddress);
            }
        }
    }

    if(getkeystring("Default", "AccessPolicy",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        number = atoi(strtmp);

        if( (number >=0) || (number <= 2))
            ComboBoxAccessPolicy->setCurrentItem( number );

        if( ComboBoxAccessPolicy->currentItem() == 0 )
        { //Disable
            AccessPushButtonAdd->setEnabled( FALSE );
            AccessPushButtonRemoveAll->setEnabled( FALSE );
            AccessPushButtonDelete->setEnabled( FALSE );
            LineEditMacAddress->setEnabled( FALSE );
            ListViewAccessControl->setEnabled( FALSE );
        }
        else
        {
            AccessPushButtonRemoveAll->setEnabled( TRUE );
            AccessPushButtonDelete->setEnabled( TRUE );
            LineEditMacAddress->setEnabled( TRUE );
            ListViewAccessControl->setEnabled( TRUE );
            if(ListViewAccessControl->childCount() >= 32)
                AccessPushButtonAdd->setEnabled( FALSE );
            else
                AccessPushButtonAdd->setEnabled( TRUE );
        }
    }

    if(getkeystring("Default", "WdsList",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        QListViewItem     *Item=NULL;
        QString           strAddress;
        unsigned char     macAddress[ETH_LENGTH_OF_ADDRESS];
        char              *macptr;
        int               i;

        for (macptr = strtok(strtmp,";"); macptr; macptr = strtok(NULL,";")) 
        {
            if(strlen(macptr) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
                continue; 

            for (i=0; i<ETH_LENGTH_OF_ADDRESS; i++)
            {
                AtoH(macptr, &macAddress[i], 2);
                macptr=macptr+3;
            }

            strAddress.sprintf("%02X-%02X-%02X-%02X-%02X-%02X", 
                macAddress[0], macAddress[1], macAddress[2], 
                macAddress[3], macAddress[4], macAddress[5]);

            Item = ListViewWds->findItem(strAddress, 0);
            if(!Item)
            {//not found.
                Item = new QListViewItem(ListViewWds, 0);
                Item->setText(0, strAddress);
            }
        }
    }

    if(getkeystring("Default", "WdsEnable",  strtmp, 255, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA) == TRUE)
    {
        int bEnable;

        bEnable = atoi(strtmp);
        if (bEnable != 0)
            bEnable = TRUE;
        else
            bEnable = FALSE;

        RadioButtonWdsEnable->setChecked(bEnable);
        WdsPage_EnableControl(bEnable);
    }
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ApConfigForm::~ApConfigForm()
{
    // no need to delete child widgets, Qt does it all for us
}

void ApConfigForm::ConfigPage_OnApplyNow()
{
    // TODO: Add your control notification handler code here
    m_strSSID=LineEditSsid->text();

    if(m_strSSID.isEmpty())
    {
        QMessageBox *mb = new QMessageBox();
        mb->setCaption("Warning");
        mb->setText("Must input SSID!");
        mb->setIcon(QMessageBox::Critical);
        mb->setButtonText(0, "&Ok");
        mb->exec();
        delete mb;

        return;
    }
    ConfigPage_WriteConfigToRegistryAndSetOid();
}

void ApConfigForm::ConfigPage_OnSecurity()
{
    m_pdlgAuthSecu->exec();
    
    if (m_pdlgAuthSecu->IsClickWepOkButton() == FALSE)
    {
        delete m_pdlgAuthSecu;
        m_pdlgAuthSecu = new AuthSecuDlg(this, "Authentication && Security", TRUE);
    }
}

bool ApConfigForm::ConfigPage_WriteConfigToRegistryAndSetOid()
{
    NDIS_802_11_RATES                       aryRates;
    NDIS_802_11_CONFIGURATION               Configuration;
    NDIS_802_11_ANTENNA                     lAntenna;
    RT_802_11_PREAMBLE                      enumPreamType;
    NDIS_802_11_RTS_THRESHOLD               lRtsThre;
    NDIS_802_11_FRAGMENTATION_THRESHOLD     lFragThre;
    RT_802_11_AP_CONFIG                     apConfig;
    NDIS_802_11_AUTHENTICATION_MODE         enumAuthenType = Ndis802_11AuthModeOpen;
    NDIS_802_11_ENCRYPTION_STATUS           enumEncryp = Ndis802_11EncryptionDisabled;
    RT_802_11_WPA_REKEY                     wpaRekey;
    QString                                 strTmp;
    ULONG                                   lAntennaNum = 2;
    char                                    msg[255];
    char                                    strcmd[255];
    int                                     i;

    m_strSSID=LineEditSsid->text();
    writekeystring("Default", "SSID", (char *)m_strSSID.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    m_nWirelessMode = ComboBoxMode->currentItem();
    if(!G_bSupportAMode && m_nWirelessMode >= PHY_11A)
        m_nWirelessMode = PHY_11BG_MIXED;

    sprintf(msg, "%d", m_nWirelessMode);
    writekeystring("Default", "WirelessMode", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    ULONG lOldWirelessMode;
    ULONG lWirelessMode = m_nWirelessMode;

    OidQueryInformation(RT_OID_802_11_PHY_MODE, m_nSocket_Id, m_strDevice_Name, &lOldWirelessMode, sizeof(lOldWirelessMode));

    if(lOldWirelessMode != lWirelessMode)
        OidSetInformation(RT_OID_802_11_PHY_MODE, m_nSocket_Id, m_strDevice_Name, &lWirelessMode, sizeof(lWirelessMode));

    m_nTXRate = ComboBoxTxRate->currentItem();
    if((m_nWirelessMode == PHY_11B && m_nTXRate > 4) || (m_nWirelessMode == PHY_11A && m_nTXRate > 8))
        m_nTXRate = 0;
    sprintf(msg, "%d", m_nTXRate);
    writekeystring("Default", "TxRate", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    // TX Rate
    memset(&aryRates, 0x00, sizeof(NDIS_802_11_RATES));
    switch (lWirelessMode)
    {
        case PHY_11A: // A only
            switch (m_nTXRate)
            {
                case 1:
                    aryRates[0] = 0x0c; // 6M
                    break;
                case 2:
                    aryRates[0] = 0x12; // 9M
                    break;
                case 3:
                    aryRates[0] = 0x18; // 12M
                    break;
                case 4:
                    aryRates[0] = 0x24; // 18M
                    break;
                case 5:
                    aryRates[0] = 0x30; // 24M
                    break;
                case 6:
                    aryRates[0] = 0x48; // 36M
                    break;
                case 7:
                    aryRates[0] = 0x60; // 48M
                    break;
                case 8:
                    aryRates[0] = 0x6c; // 54M
                    break;
                case 0:
                default:
                    aryRates[0] = 0x6c; // 54Mbps
                    aryRates[1] = 0x60; // 48Mbps
                    aryRates[2] = 0x48; // 36Mbps
                    aryRates[3] = 0x30; // 24Mbps
                    aryRates[4] = 0x24; // 18M
                    aryRates[5] = 0x18; // 12M
                    aryRates[6] = 0x12; // 9M
                    aryRates[7] = 0x0c; // 6M
                    break;
            }
            break;
        case PHY_11BG_MIXED: // B/G Mixed
        case PHY_11B: // B only
        case PHY_11G: // G Only
        case PHY_11ABG_MIXED: // A/B/G Mixed
        default:
            switch (m_nTXRate)
            {
                case 0:
                    switch (lWirelessMode)
                    {
                        case PHY_11B: // B Only
                            aryRates[0] = 0x16; // 11Mbps
                            aryRates[1] = 0x0b; // 5.5Mbps
                            aryRates[2] = 0x04; // 2Mbps
                            aryRates[3] = 0x02; // 1Mbps
                            break;
                        /*
                        case PHY_11A: // A Only
                            break;
                        */
                        case PHY_11G: // G Only
                        case PHY_11BG_MIXED: // B/G Mixed
                        case PHY_11ABG_MIXED: // A/B/G Mixed
                        default:
                            aryRates[0] = 0x6c; // 54Mbps
                            aryRates[1] = 0x60; // 48Mbps
                            aryRates[2] = 0x48; // 36Mbps
                            aryRates[3] = 0x30; // 24Mbps
                            aryRates[4] = 0x16; // 11Mbps
                            aryRates[5] = 0x0b; // 5.5Mbps
                            aryRates[6] = 0x04; // 2Mbps
                            aryRates[7] = 0x02; // 1Mbps
                            break;
                    }
                    break;
                case 1:
                    aryRates[0] = 0x02; // 1M
                    break;
                case 2:
                    aryRates[0] = 0x04; // 2M
                    break;
                case 3:
                    aryRates[0] = 0x0b; // 5.5M
                    break;
                case 4:
                    aryRates[0] = 0x16; // 11M
                    break;
                case 5:
                    aryRates[0] = 0x0c; // 6M
                    break;
                case 6:
                    aryRates[0] = 0x12; // 9M
                    break;
                case 7:
                    aryRates[0] = 0x18; // 12M
                    break;
                case 8:
                    aryRates[0] = 0x24; // 18M
                    break;
                case 9:
                    aryRates[0] = 0x30; // 24M
                    break;
                case 10:
                    aryRates[0] = 0x48; // 36M
                    break;
                case 11:
                    aryRates[0] = 0x60; // 48M
                    break;
                case 12:
                    aryRates[0] = 0x6c; // 54M
                    break;
            }
            break;
    }

    switch (m_nTXRate) 
    {
        case 0: // Auto
            switch(m_nWirelessMode)
            {
                case 0: // B/G Mixed
                    aryRates[0] = 0x6c; // 54Mbps
                    aryRates[1] = 0x60; // 48Mbps
                    aryRates[2] = 0x48; // 36Mbps
                    aryRates[3] = 0x30; // 24Mbps
                    aryRates[4] = 0x16; // 11Mbps
                    aryRates[5] = 0x0b; // 5.5Mbps
                    aryRates[6] = 0x04; // 2Mbps
                    aryRates[7] = 0x02; // 1Mbps
                    break;
                case 1: // B Only
                    aryRates[0] = 0x16; // 11Mbps
                    aryRates[1] = 0x0b; // 5.5Mbps
                    aryRates[2] = 0x04; // 2Mbps
                    aryRates[3] = 0x02; // 1Mbps
                    break;
            }
            break;
        case 1:
            aryRates[0] = 0x02; // 1M   Mandatory
            break;
        case 2:
            aryRates[0] = 0x04; // 2M   Mandatory
            break;
        case 3:
            aryRates[0] = 0x0b; // 5.5M Mandatory
            break;
        case 4:
            aryRates[0] = 0x16; // 11M Mandatory
            break;
        case 5:
            aryRates[0] = 0x0c; // 6M
            break;
        case 6:
            aryRates[0] = 0x12; // 9M
            break;
        case 7:
            aryRates[0] = 0x18; // 12M
            break;
        case 8:
            aryRates[0] = 0x24; // 18M
            break;
        case 9:
            aryRates[0] = 0x30; // 24M
            break;
        case 10:
            aryRates[0] = 0x48; // 36M
            break;
        case 11:
            aryRates[0] = 0x60; // 48M
            break;
        case 12:
            aryRates[0] = 0x6c; // 54M
            break;
    }

    OidSetInformation(OID_802_11_DESIRED_RATES, m_nSocket_Id, m_strDevice_Name, &aryRates, sizeof(aryRates));

    // Configuration
    NDIS_802_11_CONFIGURATION oldConfiguration;

    m_strChannel = ComboBoxChannel->currentText();
    writekeystring("Default", "Channel", (char *)m_strChannel.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    ULONG lChannel = m_strChannel.toLong();

    OidQueryInformation(OID_802_11_CONFIGURATION, m_nSocket_Id, m_strDevice_Name, &Configuration, sizeof(Configuration));

    memcpy(&oldConfiguration, &Configuration, sizeof(NDIS_802_11_CONFIGURATION));

    if(!m_bAutoChanSel)
    {
        for(i = 0; i < G_nChanFreqCount; i++)
        {
            if(lChannel == ChannelFreqTable[i].lChannel)
            {
                Configuration.DSConfig = ChannelFreqTable[i].lFreq;
                break;
            }
        }
    }

    m_nBeaconPeriod = SpinBoxBeaconPeriod->value();
    sprintf(msg, "%d", m_nBeaconPeriod);
    writekeystring("Default", "BeaconPeriod", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    Configuration.BeaconPeriod = m_nBeaconPeriod;

    if(memcmp(&oldConfiguration, &Configuration, sizeof(NDIS_802_11_CONFIGURATION)) != 0)
        OidSetInformation(OID_802_11_CONFIGURATION, m_nSocket_Id, m_strDevice_Name, &Configuration, sizeof(Configuration));

    // TX Preamble
    m_nTXPreamble = ComboBoxTXPreamble->currentItem();
    sprintf(msg, "%d", m_nTXPreamble);
    writekeystring("Default", "TxPreamble", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    if(m_nTXPreamble == Rt802_11PreambleLong)
        enumPreamType = Rt802_11PreambleLong;
    else 
        enumPreamType = Rt802_11PreambleShort;

    RT_802_11_PREAMBLE enumOldPreamType;

    OidQueryInformation(RT_OID_802_11_PREAMBLE, m_nSocket_Id, m_strDevice_Name, &enumOldPreamType, sizeof(enumOldPreamType));
    if( enumOldPreamType != enumPreamType)
        OidSetInformation(RT_OID_802_11_PREAMBLE, m_nSocket_Id, m_strDevice_Name, &enumPreamType, sizeof(enumPreamType));

    // Rts Threshold
    m_nRtsThreshold = SpinBoxRtsThreshold->value();
    sprintf(msg, "%d", m_nRtsThreshold);
    writekeystring("Default", "RTSThreshold", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    lRtsThre = m_nRtsThreshold;
    NDIS_802_11_RTS_THRESHOLD lOldRtsThre;

    OidQueryInformation(OID_802_11_RTS_THRESHOLD, m_nSocket_Id, m_strDevice_Name, &lOldRtsThre, sizeof(lOldRtsThre));
    if(lOldRtsThre != lRtsThre)
        OidSetInformation(OID_802_11_RTS_THRESHOLD, m_nSocket_Id, m_strDevice_Name, &lRtsThre, sizeof(lRtsThre));


    // Frag Threshold
    m_nFragThreshold = SpinBoxFragThreshold->value();
    sprintf(msg, "%d", m_nFragThreshold);
    writekeystring("Default", "FragThreshold", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    lFragThre = m_nFragThreshold;
    NDIS_802_11_FRAGMENTATION_THRESHOLD lOldFragThre;

    OidQueryInformation(OID_802_11_FRAGMENTATION_THRESHOLD, m_nSocket_Id, m_strDevice_Name, &lOldFragThre, sizeof(lOldFragThre));
    if(lOldFragThre != lFragThre)
        OidSetInformation(OID_802_11_FRAGMENTATION_THRESHOLD, m_nSocket_Id, m_strDevice_Name, &lFragThre, sizeof(lFragThre));

    // AP_CONFIG
    RT_802_11_AP_CONFIG apOldConfig;

    OidQueryInformation(RT_OID_802_11_AP_CONFIG, m_nSocket_Id, m_strDevice_Name, &apConfig, sizeof(apConfig));
    memcpy(&apOldConfig, &apConfig, sizeof(RT_802_11_AP_CONFIG));
    m_nBGProtection = ComboBoxBGProtection->currentItem();
    m_bTXBurst = CheckBoxTXBurst->isChecked();
    m_bTurboRate = CheckBoxTurboRate->isChecked();
    m_bNoForwarding = CheckBoxNoForwarding->isChecked();
    m_bHideSSID = CheckBoxHideSSID->isChecked();
    m_bShortSlot = CheckBoxShortSlot->isChecked();

    if(m_nWirelessMode == PHY_11B || m_nWirelessMode == PHY_11A || m_nWirelessMode == PHY_11G)
        m_nBGProtection = FALSE;
    sprintf(msg, "%d", m_nBGProtection);
    writekeystring("Default", "BGProtection", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    sprintf(msg, "%d", m_bTXBurst);
    writekeystring("Default", "TxBurst", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    sprintf(msg, "%d", m_bTurboRate);
    writekeystring("Default", "TurboRate", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    sprintf(msg, "%d", m_bNoForwarding);
    writekeystring("Default", "NoForwarding", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    sprintf(msg, "%d", m_bHideSSID);
    writekeystring("Default", "HideSSID", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    sprintf(msg, "%d", m_bShortSlot);
    writekeystring("Default", "ShortSlot", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    m_bAutoChanSel = CheckBoxAutoSelChannel->isChecked();
    sprintf(msg, "%d", m_bAutoChanSel);
    writekeystring("Default", "AutoChannelSelect", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    apConfig.UseBGProtection = m_nBGProtection;
    apConfig.EnableTxBurst = m_bTXBurst;
    apConfig.EnableTurboRate = m_bTurboRate;
    apConfig.IsolateInterStaTraffic = m_bNoForwarding;
    apConfig.HideSsid = m_bHideSSID;
    apConfig.UseShortSlotTime = m_bShortSlot;
    if(memcmp(&apOldConfig, &apConfig, sizeof(RT_802_11_AP_CONFIG)) != 0)
        OidSetInformation(RT_OID_802_11_AP_CONFIG, m_nSocket_Id, m_strDevice_Name, &apConfig, sizeof(apConfig));

    // Authtication Type
    enumAuthenType = m_pdlgAuthSecu->Get_Wep_AuthenType();

    NDIS_802_11_AUTHENTICATION_MODE enumOldAuthenType; 

    OidQueryInformation(OID_802_11_AUTHENTICATION_MODE, m_nSocket_Id, m_strDevice_Name, &enumOldAuthenType, sizeof(enumOldAuthenType));
    if(enumAuthenType != enumOldAuthenType)
        OidSetInformation(OID_802_11_AUTHENTICATION_MODE, m_nSocket_Id, m_strDevice_Name, &enumAuthenType, sizeof(enumAuthenType));

    // Encry Type
    if ((enumAuthenType == Ndis802_11AuthModeOpen) || (enumAuthenType == Ndis802_11AuthModeShared) || 
        (enumAuthenType == Ndis802_11AuthModeAutoSwitch)) // Soft-AP use AutoSwitch to define 802_1X
    {
            enumEncryp = m_pdlgAuthSecu->Get_Wep_EncryType();
    }
    else if ((enumAuthenType == Ndis802_11AuthModeWPA) || (enumAuthenType == Ndis802_11AuthModeWPAPSK)) 
    {
        if(G_bWpaCapable)
        {
            if ((m_pdlgAuthSecu->Get_Wep_EncryType() == Ndis802_11Encryption2Enabled) && G_encryCapa >= Ndis802_11Encryption2Enabled)
                enumEncryp = Ndis802_11Encryption2Enabled; // TKIP
            else if(m_pdlgAuthSecu->Get_Wep_EncryType() == Ndis802_11Encryption3Enabled && G_encryCapa >= Ndis802_11Encryption3Enabled)
                enumEncryp = Ndis802_11Encryption3Enabled; // AES
            else
                enumEncryp = Ndis802_11Encryption2Enabled;
        }
        else
            enumEncryp = Ndis802_11EncryptionDisabled;
    }

    // Always reset WEP_STATUS
    OidSetInformation(OID_802_11_WEP_STATUS, m_nSocket_Id, m_strDevice_Name, &enumEncryp, sizeof(enumEncryp));

    if ((enumAuthenType == Ndis802_11AuthModeWPA) || (enumAuthenType == Ndis802_11AuthModeWPAPSK) 
        || (enumAuthenType == Ndis802_11AuthModeAutoSwitch)) // 802.1x
    {
        // Radius Data
        if(enumAuthenType == Ndis802_11AuthModeWPA || enumAuthenType == Ndis802_11AuthModeAutoSwitch) // 802.1x
        {
            //Save Radius Data 
            strTmp = m_pdlgAuthSecu->Get_RADIUS_Server();
            if (strTmp == NULL)
                writekeystring("Default", "RADIUS_Server", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
            else
            writekeystring("Default", "RADIUS_Server", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

            strTmp = m_pdlgAuthSecu->Get_RADIUS_Key();
            if (strTmp == NULL)
                writekeystring("Default", "RADIUS_Key", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
            else
            writekeystring("Default", "RADIUS_Key", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

            sprintf(msg, "%d", m_pdlgAuthSecu->Get_RADIUS_Port());
            writekeystring("Default", "RADIUS_Port", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
        }
        else if (enumAuthenType == Ndis802_11AuthModeWPAPSK)
        { // Pre ShareKey
            ULONG nKeyLen = 32;
            ULONG lBufLen = sizeof(NDIS_802_11_KEY) + nKeyLen - 1;
            UCHAR keyMaterial[40];
            QString strPsk;
            QString strSSID;

            // Allocate Resouce
            PNDIS_802_11_KEY pKey = (PNDIS_802_11_KEY)malloc(lBufLen);
                
            pKey->Length = lBufLen;
            pKey->KeyLength = nKeyLen;
            pKey->KeyIndex = 0x80000000;

            strPsk = m_pdlgAuthSecu->Get_WpaPsk_text();
            strSSID = LineEditSsid->text();
            int nLen = strSSID.length();

            if (strPsk.length() == 64)
                AtoH((char *)strPsk.data(), keyMaterial, 32);
            else
                PasswordHash((char *)strPsk.data(), (unsigned char *)strSSID.data(), nLen, keyMaterial);
            
            memcpy(pKey->KeyMaterial, keyMaterial, 32);
            OidSetInformation(OID_802_11_ADD_KEY, m_nSocket_Id, m_strDevice_Name, pKey, pKey->Length);
        }

        // ReKey
        if(enumAuthenType == Ndis802_11AuthModeWPA || enumAuthenType == Ndis802_11AuthModeWPAPSK)
        {
            memset(&wpaRekey, 0x00, sizeof(RT_802_11_WPA_REKEY));
            wpaRekey.ReKeyMethod = m_pdlgAuthSecu->Get_RekeyMethod();
            wpaRekey.ReKeyInterval = m_pdlgAuthSecu->Get_RekeyInterval();
            RT_802_11_WPA_REKEY wpaOldRekey;

            OidQueryInformation(RT_OID_802_11_WPA_REKEY, m_nSocket_Id, m_strDevice_Name, &wpaOldRekey, sizeof(RT_802_11_WPA_REKEY));
            if(memcmp(&wpaRekey, &wpaOldRekey, sizeof(RT_802_11_WPA_REKEY)) != 0)                
                OidSetInformation(RT_OID_802_11_WPA_REKEY, m_nSocket_Id, m_strDevice_Name, &wpaRekey, sizeof(RT_802_11_WPA_REKEY));
        }
    }

    // WepKey
    if(enumEncryp == Ndis802_11Encryption1Enabled || enumAuthenType == Ndis802_11AuthModeShared) 
    {
        strTmp = m_pdlgAuthSecu->Get_Wep_KeyString(0);
        ULONG nKeyLen = strTmp.length();
        if(nKeyLen == 0)
        {
            NDIS_802_11_REMOVE_KEY removeKey;

            removeKey.Length = sizeof(NDIS_802_11_REMOVE_KEY);
            removeKey.KeyIndex = 0;
            for(int i = 0; i < 6; i++)
                removeKey.BSSID[i] = 0xff;
            OidSetInformation(OID_802_11_REMOVE_KEY, m_nSocket_Id, m_strDevice_Name, &removeKey, removeKey.Length);
        }
        else
        {
            if(nKeyLen == 10)
                nKeyLen = 5;
            else if(nKeyLen == 26)
                nKeyLen = 13;

            ULONG lBufLen = sizeof(NDIS_802_11_KEY) + nKeyLen - 1;

            // Allocate Resource
            PNDIS_802_11_KEY pKey = (PNDIS_802_11_KEY)malloc(lBufLen);
            pKey->Length = lBufLen;
            pKey->KeyLength = nKeyLen;
            pKey->KeyIndex = 0;
            if(m_pdlgAuthSecu->Get_Wep_KeyID() == 0)
                pKey->KeyIndex |= 0x80000000;
            for(i = 0; i < 6; i++)
                pKey->BSSID[i] = 0xff;

            if(strTmp.length() == 5)
                memcpy(pKey->KeyMaterial, strTmp.data(), 5);
            else if(strTmp.length() == 10)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 5);
            else if(strTmp.length() == 13)
                memcpy(pKey->KeyMaterial, strTmp.data(), 13);
            else if(strTmp.length() == 26)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 13);

            OidSetInformation(OID_802_11_ADD_KEY, m_nSocket_Id, m_strDevice_Name, pKey, pKey->Length);
        }

        strTmp = m_pdlgAuthSecu->Get_Wep_KeyString(1);
        nKeyLen = strTmp.length();
        if(nKeyLen == 0)
        {
            NDIS_802_11_REMOVE_KEY removeKey;
            removeKey.Length = sizeof(NDIS_802_11_REMOVE_KEY);
            removeKey.KeyIndex = 1;
            for(i = 0; i < 6; i++)
                removeKey.BSSID[i] = 0xff;
            OidSetInformation(OID_802_11_REMOVE_KEY, m_nSocket_Id, m_strDevice_Name, &removeKey, removeKey.Length);
        }
        else
        {
            if(nKeyLen == 10)
                nKeyLen = 5;
            else if(nKeyLen == 26)
                nKeyLen = 13;

            ULONG lBufLen = sizeof(NDIS_802_11_KEY) + nKeyLen - 1;

            // Allocate Resource
            PNDIS_802_11_KEY pKey = (PNDIS_802_11_KEY)malloc(lBufLen);
            pKey->Length = lBufLen;
            pKey->KeyLength = nKeyLen;
            pKey->KeyIndex = 1;
            if(m_pdlgAuthSecu->Get_Wep_KeyID() == 1)
                pKey->KeyIndex |= 0x80000000;
            for(i = 0; i < 6; i++)
                pKey->BSSID[i] = 0xff;

            if(strTmp.length() == 5)
                memcpy(pKey->KeyMaterial, strTmp.data(), 5);
            else if(strTmp.length() == 10)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 5);
            else if(strTmp.length() == 13)
                memcpy(pKey->KeyMaterial, strTmp.data(), 13);
            else if(strTmp.length() == 26)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 13);

            OidSetInformation(OID_802_11_ADD_KEY, m_nSocket_Id, m_strDevice_Name, pKey, pKey->Length);
        }

        strTmp = m_pdlgAuthSecu->Get_Wep_KeyString(2);
        nKeyLen = strTmp.length();
        if(nKeyLen == 0)
        {
            NDIS_802_11_REMOVE_KEY removeKey;
            removeKey.Length = sizeof(NDIS_802_11_REMOVE_KEY);
            removeKey.KeyIndex = 2;
            for(i = 0; i < 6; i++)
                removeKey.BSSID[i] = 0xff;
            OidSetInformation(OID_802_11_REMOVE_KEY, m_nSocket_Id, m_strDevice_Name, &removeKey, removeKey.Length);
        }
        else
        {
            if(nKeyLen == 10)
                nKeyLen = 5;
            else if(nKeyLen == 26)
                nKeyLen = 13;

            ULONG lBufLen = sizeof(NDIS_802_11_KEY) + nKeyLen - 1;

            // Allocate Resource
            PNDIS_802_11_KEY pKey = (PNDIS_802_11_KEY)malloc(lBufLen);
            pKey->Length = lBufLen;
            pKey->KeyLength = nKeyLen;
            pKey->KeyIndex = 2;
            if(m_pdlgAuthSecu->Get_Wep_KeyID() == 2)
                pKey->KeyIndex |= 0x80000000;
            for(i = 0; i < 6; i++)
                pKey->BSSID[i] = 0xff;

            if(strTmp.length() == 5)
                memcpy(pKey->KeyMaterial, strTmp.data(), 5);
            else if(strTmp.length() == 10)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 5);
            else if(strTmp.length() == 13)
                memcpy(pKey->KeyMaterial, strTmp.data(), 13);
            else if(strTmp.length() == 26)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 13);

            OidSetInformation(OID_802_11_ADD_KEY, m_nSocket_Id, m_strDevice_Name, pKey, pKey->Length);
        }

        strTmp = m_pdlgAuthSecu->Get_Wep_KeyString(3);
        nKeyLen = strTmp.length();
        if(nKeyLen == 0)
        {
            NDIS_802_11_REMOVE_KEY removeKey;
            removeKey.Length = sizeof(NDIS_802_11_REMOVE_KEY);
            removeKey.KeyIndex = 3;
            for(i = 0; i < 6; i++)
                removeKey.BSSID[i] = 0xff;
            OidSetInformation(OID_802_11_REMOVE_KEY, m_nSocket_Id, m_strDevice_Name, &removeKey, removeKey.Length);
        }
        else
        {
            if(nKeyLen == 10)
                nKeyLen = 5;
            else if(nKeyLen == 26)
                nKeyLen = 13;

            ULONG lBufLen = sizeof(NDIS_802_11_KEY) + nKeyLen - 1;

            // Allocate Resource
            PNDIS_802_11_KEY pKey = (PNDIS_802_11_KEY)malloc(lBufLen);
            pKey->Length = lBufLen;
            pKey->KeyLength = nKeyLen;
            pKey->KeyIndex = 3;
            if(m_pdlgAuthSecu->Get_Wep_KeyID() == 3)
                pKey->KeyIndex |= 0x80000000;
            for(i = 0; i < 6; i++)
                pKey->BSSID[i] = 0xff;

            if(strTmp.length() == 5)
                memcpy(pKey->KeyMaterial, strTmp.data(), 5);
            else if(strTmp.length() == 10)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 5);
            else if(strTmp.length() == 13)
                memcpy(pKey->KeyMaterial, strTmp.data(), 13);
            else if(strTmp.length() == 26)
                AtoH((char *)strTmp.data(), pKey->KeyMaterial, 13);

            OidSetInformation(OID_802_11_ADD_KEY, m_nSocket_Id, m_strDevice_Name, pKey, pKey->Length);
        }
    }

    strTmp = m_pdlgAuthSecu->Get_AuthenType_text();
    writekeystring("Default", "AuthMode", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp = m_pdlgAuthSecu->Get_EncryType_text();
    writekeystring("Default", "EncrypType", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp = m_pdlgAuthSecu->Get_WpaPsk_text();
    if (strTmp == NULL)
        writekeystring("Default", "WPAPSK", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    else
        writekeystring("Default", "WPAPSK", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    
    sprintf(msg, "%d", m_pdlgAuthSecu->Get_RekeyInterval());
    writekeystring("Default", "RekeyInterval",  msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);    

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_RekeyMethod());
    writekeystring("Default", "RekeyMethod", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_Wep_KeyID() + 1);
    writekeystring("Default", "DefaultKeyID", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_Wep_KeyType(0) );
    writekeystring("Default", "Key1Type", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp=m_pdlgAuthSecu->Get_Wep_KeyString(0);
    if (strTmp == NULL)
        writekeystring("Default", "Key1Str", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    else
        writekeystring("Default", "Key1Str", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_Wep_KeyType(1) );
    writekeystring("Default", "Key2Type", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp=m_pdlgAuthSecu->Get_Wep_KeyString(1);
    if (strTmp == NULL)
        writekeystring("Default", "Key2Str", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    else
        writekeystring("Default", "Key2Str", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_Wep_KeyType(2) );
    writekeystring("Default", "Key3Type", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp=m_pdlgAuthSecu->Get_Wep_KeyString(2);
    if (strTmp == NULL)
        writekeystring("Default", "Key3Str", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    else
        writekeystring("Default", "Key3Str", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    sprintf(msg, "%d", m_pdlgAuthSecu->Get_Wep_KeyType(3) );
    writekeystring("Default", "Key4Type", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strTmp=m_pdlgAuthSecu->Get_Wep_KeyString(3);
    if (strTmp == NULL)
        writekeystring("Default", "Key4Str", "", RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);
    else
        writekeystring("Default", "Key4Str", (char *) strTmp.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    //SSID
    NDIS_802_11_SSID SSID;

    m_strSSID=LineEditSsid->text();
    SSID.SsidLength = m_strSSID.length();
    memcpy(SSID.Ssid, m_strSSID.data(), SSID.SsidLength);

    OidSetInformation(OID_802_11_SSID, m_nSocket_Id, m_strDevice_Name, &SSID, sizeof(SSID));

    return TRUE;
}

void ApConfigForm::ConfigPage_ComboBoxMode_activated(int id)
{
    ConfigPage_OnSelchangeWirelessMode();
}

void ApConfigForm::ConfigPage_OnUseMacAddress()
{
    NDIS_802_11_MAC_ADDRESS CurrentAddress;
    QString strTmp;

    memset(CurrentAddress, 0x00, sizeof(CurrentAddress));
    OidQueryInformation(OID_802_3_CURRENT_ADDRESS, m_nSocket_Id, m_strDevice_Name, &CurrentAddress, sizeof(CurrentAddress));

    strTmp.sprintf("AP%02X%02X%02X%02X%02X%02X", CurrentAddress[0], CurrentAddress[1],
                        CurrentAddress[2], CurrentAddress[3], CurrentAddress[4], CurrentAddress[5]);
    LineEditSsid->setText(strTmp);
}

void ApConfigForm::ConfigPage_OnDefault()
{
    LineEditSsid->setText( trUtf8( "" ) );
    ComboBoxMode->setCurrentItem( 0 );
    ComboBoxTxRate->setCurrentItem( 0 );
    ComboBoxBGProtection->setCurrentItem( 0 );
    ComboBoxTXPreamble->setCurrentItem( 0 );
    SpinBoxBeaconPeriod->setValue(100);
    SpinBoxFragThreshold->setValue( 2346 );
    SpinBoxRtsThreshold->setValue( 2347 );
    CheckBoxTXBurst->setChecked( FALSE );
    CheckBoxTurboRate->setChecked( FALSE );
    CheckBoxNoForwarding->setChecked( FALSE );
    CheckBoxHideSSID->setChecked( FALSE );
    CheckBoxShortSlot->setChecked( FALSE );
    CheckBoxAutoSelChannel->setChecked( FALSE );
    ComboBoxChannel->setCurrentItem(0);
    ConfigPage_OnSelchangeWirelessMode();

    delete m_pdlgAuthSecu;

    m_pdlgAuthSecu = new AuthSecuDlg(this, "Authentication && Security", TRUE);
}


void ApConfigForm::ConfigPage_OnSelchangeWirelessMode() 
{
    char channelString[3];
    int i;
    
    ComboBoxTxRate->clear();

    switch(ComboBoxMode->currentItem())
    {
        case PHY_11B: // B Only
            ComboBoxTxRate->insertItem( trUtf8( "Auto" ) );
            ComboBoxTxRate->insertItem( trUtf8( "1 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "2 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "5.5 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "11 Mbps" ) );
            ComboBoxBGProtection->setEnabled( FALSE );
            ComboBoxBGProtection->setCurrentItem( 0 );
            break;
        case PHY_11A: // A Only
            ComboBoxTxRate->insertItem( trUtf8( "Auto" ) );
            ComboBoxTxRate->insertItem( trUtf8( "6 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "9 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "12 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "18 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "24 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "36 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "48 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "54 Mbps" ) );
            ComboBoxBGProtection->setEnabled( FALSE );
            ComboBoxBGProtection->setCurrentItem( 0 );
            break;
        case PHY_11G: // G Only
        case PHY_11BG_MIXED: // B/G Mixed
        case PHY_11ABG_MIXED: // A/B/G Mixed
        default:
            ComboBoxTxRate->insertItem( trUtf8( "Auto" ) );
            ComboBoxTxRate->insertItem( trUtf8( "1 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "2 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "5.5 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "11 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "6 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "9 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "12 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "18 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "24 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "36 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "48 Mbps" ) );
            ComboBoxTxRate->insertItem( trUtf8( "54 Mbps" ) );
//          ComboBoxTxRate->insertItem( trUtf8( "72 Mbps" ) );
//          ComboBoxTxRate->insertItem( trUtf8( "100 Mbps" ) );
            if (ComboBoxMode->currentItem() == PHY_11G)
                ComboBoxBGProtection->setEnabled( FALSE );
            else
                ComboBoxBGProtection->setEnabled( TRUE );
            break;
    }
    ComboBoxTxRate->setCurrentItem( 0 );

    GetCountryChannel();
    m_strChannel = ComboBoxChannel->currentText();
    ComboBoxChannel->clear();
    switch (ComboBoxMode->currentItem())
    {
        case PHY_11ABG_MIXED: // A/B/G Mixed
            for (i = 0; i < m_n24GhzIndex; i++)
            {
                sprintf(channelString, "%d", m_chAryChan24Ghz[i]);
                ComboBoxChannel->insertItem(channelString);
            }
            break;
        case PHY_11A: // A Only
            for (i = 0; i < m_n52GhzIndex; i++)
            {
                sprintf(channelString, "%d", m_chAryChan52Ghz[i]);
                ComboBoxChannel->insertItem(channelString);
            }
            ComboBoxChannel->setCurrentItem(0);
            break;
        case PHY_11B: // B Only
        case PHY_11G: // G Only
        case PHY_11BG_MIXED: // B/G Mixed
        default:
            for (i = 0; i < m_n24GhzIndex; i++)
            {
                sprintf(channelString, "%d", m_chAryChan24Ghz[i]);
                ComboBoxChannel->insertItem(channelString);
            }
            ComboBoxChannel->setCurrentItem(0);
            break;
    }    

    QString text;
    for (i = 0; i < ComboBoxChannel->count(); i++)
    {
        text = ComboBoxChannel->text(i);
        if (text == m_strChannel)
        {
            ComboBoxChannel->setCurrentItem( i );
            break;
        }
    }

    if (i >= ComboBoxChannel->count())
    {//not found
        ComboBoxChannel->setCurrentItem( 0 );
    }
}

void ApConfigForm::OnPageChanged()
{
    killTimer( timerId_getMacTable );
    timerId_getMacTable = -1;
    killTimer( timerId_getEventLog );
    timerId_getEventLog = -1;
    killTimer( timerId_getStatistics);
    timerId_getStatistics = -1;

    switch(TabWidgetApConfigForm->currentPageIndex())
    {
        case 0: //Config Page
            break;
        case 1: //AccessControl Page
            break;
        case 2: //Link Status Page
            WdsPage_OnActive();
            break;
        case 3: //Mac Table Page
            MacTablePage_OnActive();
            break;
        case 4: //Event Log Page
            EventLogPage_OnActive();
            break;
        case 5: //Statistics Page
            StatisticsPage_OnActive();
            break;
        case 6: //About
            AboutPage_OnActive();
            break;
        default :
            break;
    }
}

void ApConfigForm::AccessControlPage_OnApplyNow()
{
    AccessControlPage_WriteConfigToRegistryAndSetOid();
}

bool ApConfigForm::AccessControlPage_WriteConfigToRegistryAndSetOid()
{
    QListViewItem       *viewItem;
    QString             str;
    QString             slist;
    RT_802_11_ACL       acl;
    char                msg[255];

    memset(&acl, 0x00, sizeof(RT_802_11_ACL));

    acl.Policy = ComboBoxAccessPolicy->currentItem();
    sprintf(msg, "%d", acl.Policy);
    writekeystring("Default", "AccessPolicy", msg, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    acl.Num = ListViewAccessControl->childCount();

    viewItem = ListViewAccessControl->firstChild();
    for(ULONG i = 0; i < acl.Num; i++)
    {
        str=viewItem->text(0);
        int j = 0;
        while ( ( j = str.find( '-', j ) ) != -1 )
            str.remove( j ,1 );

        AtoH((char *)str.data(), acl.Entry[i].Addr, 6);
        str.sprintf("%02X:%02X:%02X:%02X:%02X:%02X;", 
            acl.Entry[i].Addr[0], acl.Entry[i].Addr[1], acl.Entry[i].Addr[2], 
            acl.Entry[i].Addr[3], acl.Entry[i].Addr[4], acl.Entry[i].Addr[5]);
        slist.append(str);
        viewItem = viewItem->itemBelow();
    }

    writekeystring("Default", "AccessControlList", (char *)slist.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    if(acl.Policy == 0 || acl.Num == 0)
    {
        acl.Policy = 0;
        acl.Num = 0;
    }

    OidSetInformation(RT_OID_802_11_ACL, m_nSocket_Id, m_strDevice_Name, &acl, sizeof(acl));
    return TRUE;
}

void ApConfigForm::AccessControlPage_OnAdd()
{
    QListViewItem  *Item=NULL;

    m_strMacAddress = LineEditMacAddress->text();

    if(m_strMacAddress.length() != 12)
    {
        QMessageBox *mb = new QMessageBox();
        mb->setCaption("Warning");
        mb->setText("Invalid Mac Address.\n eq. 0c00124304c8");
        mb->setIcon(QMessageBox::Critical);
        mb->setButtonText(0, "&Ok");
        mb->exec();
        delete mb;

        return;
    }

    if (m_strMacAddress.compare("000000000000") == 0)
    {
        QMessageBox::warning(this, "Warning", "Can't accept 00-00-00-00-00-00 Mac Address");
        return;
    }

    UCHAR chMulticast;
    AtoH((char *)m_strMacAddress.data(), &chMulticast, 1);

    if(chMulticast & 0x01)
    {
        QMessageBox::warning(this, "Warning", "Can't accept multicast/broadcast Mac Address");
        return;
    }

    for(int i = 2; i < 17; i += 3)
        m_strMacAddress.insert(i, '-');

    Item = ListViewAccessControl->findItem(m_strMacAddress, 0);
    if(!Item)
    {//not found.
        Item = new QListViewItem(ListViewAccessControl, 0);
        Item->setText(0, m_strMacAddress);
        ListViewAccessControl->setCurrentItem(Item);
    }
    else
    {//Duplicate

        QMessageBox *mb = new QMessageBox();
        mb->setCaption("Warning");
        m_strMacAddress.insert(0, "Duplicate Mac Address.\n");
        mb->setText( m_strMacAddress );
        mb->setIcon(QMessageBox::Critical);
        mb->setButtonText(0, "&Ok");
        mb->exec();
        delete mb;

        return;
    }
    LineEditMacAddress->clear();

    if(ListViewAccessControl->childCount() >= 32)
        AccessPushButtonAdd->setEnabled( FALSE );
}

void ApConfigForm::AccessControlPage_OnDelete()
{
    QListViewItem *Item;

    Item = ListViewAccessControl->currentItem();
    if(Item == NULL)
        return ;

    ListViewAccessControl->takeItem(Item);
    AccessPushButtonAdd->setEnabled( TRUE );
}

void ApConfigForm::AccessControlPage_OnRemoveAll()
{
    ListViewAccessControl->clear();
    AccessPushButtonAdd->setEnabled( TRUE );
}

void ApConfigForm::AccessControlPage_OnSelchangeAccessPolicy(int id)
{
    if(id == 0)
    {
        AccessPushButtonAdd->setEnabled( FALSE );
        AccessPushButtonRemoveAll->setEnabled( FALSE );
        AccessPushButtonDelete->setEnabled( FALSE );
        LineEditMacAddress->setEnabled( FALSE );
        ListViewAccessControl->setEnabled( FALSE );
    }
    else
    {
        AccessPushButtonRemoveAll->setEnabled( TRUE );
        AccessPushButtonDelete->setEnabled( TRUE );
        LineEditMacAddress->setEnabled( TRUE );
        ListViewAccessControl->setEnabled( TRUE );
        if(ListViewAccessControl->childCount() >= 32)
            AccessPushButtonAdd->setEnabled( FALSE );
        else
            AccessPushButtonAdd->setEnabled( TRUE );
    }
}

void ApConfigForm::WdsPage_OnActive()
{
    if (RadioButtonWdsEnable->isChecked()) //Enable
        WdsPage_EnableControl( TRUE );
    else
        WdsPage_EnableControl( FALSE );
}

void ApConfigForm::timerEvent(QTimerEvent* e)
{
    NDIS_802_11_MAC_ADDRESS        CurrentAddress;

    if(e->timerId() == timerId_alive)
    {        
        if(OidQueryInformation(OID_802_3_CURRENT_ADDRESS, m_nSocket_Id, m_strDevice_Name, &CurrentAddress, sizeof(CurrentAddress)) == -1 &&
            errno == ENODEV) //No such device from errno.h)
        {
            killTimers();
            close();
        }
    }
    else if(e->timerId() == timerId_getMacTable)
    {
        MacTablePage_GetMacTable();
    }
    else if(e->timerId() == timerId_getEventLog)
    {
        EventLogPage_GetEvent();
    }
    else if(e->timerId() == timerId_getStatistics)
    {
        StatisticsPage_GetStatistics();
    }
}

void ApConfigForm::MacTablePage_OnActive()
{
    timerId_getMacTable = startTimer(2000);
    MacTablePage_GetMacTable();
}

void ApConfigForm::MacTablePage_GetMacTable()
{
    RT_802_11_MAC_TABLE macTable;
    QListViewItem  *ListItem=NULL;
    QString strMacAddr, strAid;

    memset(&macTable, 0x00, sizeof(RT_802_11_MAC_TABLE));

    if( OidQueryInformation(RT_OID_802_11_MAC_TABLE, m_nSocket_Id, m_strDevice_Name, &macTable, sizeof(macTable)) == 0) 
    {//success
        ListViewMacTable->clear();
        for(ULONG i = 0; i < macTable.Num; i++)
        {
            ListItem=new QListViewItem(ListViewMacTable, 0);

            strMacAddr.sprintf(" %02X-%02X-%02X-%02X-%02X-%02X", macTable.Entry[i].Addr[0], macTable.Entry[i].Addr[1],
                        macTable.Entry[i].Addr[2], macTable.Entry[i].Addr[3], macTable.Entry[i].Addr[4], macTable.Entry[i].Addr[5]);
            strAid.sprintf("  %d  ", macTable.Entry[i].Aid);
            ListItem->setText(0, strMacAddr);
            ListItem->setText(1, strAid);

            if(macTable.Entry[i].Psm)
                ListItem->setText(2, "  Yes  ");
            else
                ListItem->setText(2, "  No  ");
        }
    }
}

void ApConfigForm::EventLogPage_OnActive()
{
    timerId_getEventLog = startTimer(2000);
    EventLogPage_GetEvent();
}

void ApConfigForm::EventLogPage_GetEvent()
{
    QListViewItem               *Item;
    char                        tmp[100], strMacAddr[50], strEventTime[50];
    RT_802_11_EVENT_LOG         eventLog;
    ULONG                       i, nListCount, nArySize;
    RT_802_11_EVENT_TABLE       eventTable;
    int                         yearp, monp, dayp, hourp, minp, secp;

    memset(&eventTable, 0, sizeof(RT_802_11_EVENT_TABLE));

    if(OidQueryInformation(RT_OID_802_11_EVENT_TABLE, m_nSocket_Id, m_strDevice_Name, &eventTable, sizeof(eventTable)) == 0)
    {
        nArySize = G_aryEventLog.size() + eventTable.Num;

        if(nArySize > MAX_ARRAY_SIZE_OF_EVENT)
        {
            G_aryEventLog.erase(G_aryEventLog.begin(), G_aryEventLog.at(nArySize - MAX_ARRAY_SIZE_OF_EVENT)) ;
            G_nEventRead -= nArySize - MAX_ARRAY_SIZE_OF_EVENT;
            if(G_nEventRead < 0)
                G_nEventRead = 0;
        }

        for(i = 0; i < eventTable.Num; i++)
        {
            G_aryEventLog.append (eventTable.Log[i]);
        }

        nArySize = G_aryEventLog.size();

        for(i = G_nEventRead; i < nArySize; i++)
        {
            Item = new QListViewItem(ListViewEventLog, 0);

            eventLog = G_aryEventLog[i];
            unmktime(eventLog.SystemTime, 0, &yearp, &monp, &dayp, &hourp, &minp, &secp);

            sprintf(strEventTime, " %04d / %02d / %02d - %02d : %02d : %02d ",
                    yearp, monp, dayp, hourp, minp, secp);
            Item->setText( 0, strEventTime);

            sprintf(strMacAddr, " %02X-%02X-%02X-%02X-%02X-%02X ", eventLog.Addr[0], eventLog.Addr[1],
                    eventLog.Addr[2], eventLog.Addr[3], eventLog.Addr[4], eventLog.Addr[5]);

            switch(eventLog.Event)
            {
                case EVENT_RESET_ACCESS_POINT:
                    sprintf(tmp, " Restart Access Point ");
                    break;
                case EVENT_ASSOCIATED:
                    sprintf(tmp, " STA %s associated ", strMacAddr);
                    break;
                case EVENT_DISASSOCIATED:
                    sprintf(tmp, " STA %s left this BSS ", strMacAddr);
                    break;
                case EVENT_AGED_OUT:
                    sprintf(tmp, " STA %s was aged-out and removed from this BSS ", strMacAddr);
                    break;
                case EVENT_COUNTER_M:
                    sprintf(tmp, " STA %s generated active countermeasure", strMacAddr);
                    break;
                case EVENT_INVALID_PSK:
                    sprintf(tmp, " STA %s associated with invalid PSK password", strMacAddr);
                    break;
                case EVENT_MAX_EVENT_TYPE:
                default:
                    break;
            }
            Item->setText( 1, tmp);
        }

        G_nEventRead = i;

        nListCount = ListViewEventLog->childCount();
        if(nListCount > MAX_ARRAY_SIZE_OF_EVENT)
        {
            for (i = 0; i < nListCount - MAX_ARRAY_SIZE_OF_EVENT; i++)
            {
                ListViewEventLog->takeItem(ListViewEventLog->firstChild());
            }
        }
    }
}

void ApConfigForm::EventLogPage_OnClear()
{
    G_aryEventLog.clear();
    ListViewEventLog->clear();
    G_nEventRead = 0;
}

void ApConfigForm::StatisticsPage_OnActive()
{
    timerId_getStatistics = startTimer(2000);
    StatisticsPage_GetStatistics();
}

void ApConfigForm::StatisticsPage_GetStatistics()
{
    NDIS_802_11_STATISTICS Statistics;
    char tmp[40];

    if( OidQueryInformation(OID_802_11_STATISTICS, m_nSocket_Id, m_strDevice_Name, &Statistics, sizeof(Statistics)) == 0 )
    {
        sprintf(tmp, "%8lld", Statistics.TransmittedFragmentCount);
        TranSucFrameLable->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.TransmittedFragmentCount - Statistics.RetryCount);
        TranSucWithoutRetryLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.RetryCount);
        TranSucAftRetryLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.FailedCount);
        TranFailAckLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.RTSSuccessCount);
        TranSucRTSLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.RTSFailureCount);
        TranFailRTSLabel->setText(tmp);
    }

    ULONG lRcvOk = 0;

    if( OidQueryInformation(OID_GEN_RCV_OK, m_nSocket_Id, m_strDevice_Name, &lRcvOk, sizeof(lRcvOk)) == 0 )
    {
        sprintf(tmp, "%8ld", lRcvOk);
        RcvSucFrameLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.FCSErrorCount);
        RcvCRCErrLabel->setText(tmp);
    }

    ULONG lRcvNoBuf = 0;

    if( OidQueryInformation(OID_GEN_RCV_NO_BUFFER, m_nSocket_Id, m_strDevice_Name, &lRcvNoBuf, sizeof(lRcvNoBuf)) == 0 )
    {
        sprintf(tmp, "%8ld", lRcvNoBuf);
        RcvDropFrameLabel->setText(tmp);
        sprintf(tmp, "%8lld", Statistics.FrameDuplicateCount);
        RcvDupFrameLabel->setText(tmp);
    }
}

void ApConfigForm::StatisticsPage_OnResetCounters()
{
    TranSucFrameLable->setText(0);
    TranSucWithoutRetryLabel->setText(0);
    TranSucAftRetryLabel->setText(0);
    TranFailAckLabel->setText(0);
    TranSucRTSLabel->setText(0);
    TranFailRTSLabel->setText(0);
    RcvSucFrameLabel->setText(0);
    RcvCRCErrLabel->setText(0);
    RcvDropFrameLabel->setText(0);
    RcvDupFrameLabel->setText(0);

    OidSetInformation(RT_OID_802_11_RESET_COUNTERS, m_nSocket_Id, m_strDevice_Name, 0, 0);
}

void ApConfigForm::AboutPage_OnActive()
{
    char tmp[40];
    NDIS_802_11_MAC_ADDRESS        CurrentAddress;
    RT_VERSION_INFO                DriverVersionInfo;

    if( OidQueryInformation(OID_802_3_CURRENT_ADDRESS, m_nSocket_Id, m_strDevice_Name, &CurrentAddress, sizeof(CurrentAddress)) == 0)
    {
        sprintf(tmp, "%02X-%02X-%02X-%02X-%02X-%02X", CurrentAddress[0], CurrentAddress[1],
                    CurrentAddress[2], CurrentAddress[3], CurrentAddress[4], CurrentAddress[5]);
        MacTextLabel->setText(tmp);
    }

    if( OidQueryInformation(RT_OID_VERSION_INFO, m_nSocket_Id, m_strDevice_Name, &DriverVersionInfo, sizeof(DriverVersionInfo)) == 0)
    {
        sprintf(tmp, "%d.%d.%d", DriverVersionInfo.DriverMajorVersion, DriverVersionInfo.DriverMinorVersion,
                    DriverVersionInfo.DriverSubVersion);
        NicVersionTextLabel->setText(tmp);

        sprintf(tmp, "%04d-%02d-%02d", DriverVersionInfo.DriverBuildYear, DriverVersionInfo.DriverBuildMonth,
                    DriverVersionInfo.DriverBuildDay);
        NicDateTextLabel->setText(tmp);
    }
}

void ApConfigForm::WdsPage_ButtonGroupWds_Click(int id)
{
    if (id == 0)
    {//enable
        WdsPage_EnableControl( TRUE );
    }
    else
    {//disable
        WdsPage_EnableControl( FALSE );
    }
}


void ApConfigForm::WdsPage_OnApplyNow()
{
    WdsPage_WriteConfigToRegistryAndSetOid();
}

void ApConfigForm::WdsPage_OnAdd()
{
    QListViewItem  *Item=NULL;
    QString strWdsMacAddress;

    
    strWdsMacAddress = LineEditWdsMacAddress->text();

    if (strWdsMacAddress.length() != 12)
    {
        QMessageBox *mb = new QMessageBox();
        mb->setCaption("Warning");
        mb->setText("Invalid Mac Address.\n eq. 0c00124304c8");
        mb->setIcon(QMessageBox::Critical);
        mb->setButtonText(0, "&Ok");
        mb->exec();
        delete mb;

        return;
    }

    if (strWdsMacAddress.compare("000000000000") == 0)
    {
        QMessageBox::warning(this, "Warning", "Can't accept 00-00-00-00-00-00 Mac Address");
        return;
    }

    UCHAR chMulticast;
    AtoH((char *)strWdsMacAddress.data(), &chMulticast, 1);

    if(chMulticast & 0x01)
    {
        QMessageBox::warning(this, "Warning", "Can't accept multicast/broadcast Mac Address");
        return;
    }

    for (int i = 2; i < 17; i += 3)
        strWdsMacAddress.insert(i, '-');

    Item = ListViewWds->findItem(strWdsMacAddress, 0);
    if (!Item)
    {//not found.
        Item = new QListViewItem(ListViewWds, 0);
        Item->setText(0, strWdsMacAddress);
        ListViewWds->setCurrentItem(Item);
    }
    else
    {//Duplicate

        QMessageBox *mb = new QMessageBox();
        mb->setCaption("Warning");
        m_strMacAddress.insert(0, "Duplicate Mac Address.\n");
        //mb->setText("Duplicate Mac Address.\n");
        mb->setText( strWdsMacAddress );
        mb->setIcon(QMessageBox::Critical);
        mb->setButtonText(0, "&Ok");
        mb->exec();
        delete mb;

        return;
    }
    LineEditWdsMacAddress->clear();

    if (ListViewWds->childCount() >= 3)
        WdsPushButtonAdd->setEnabled( FALSE );
    
}

void ApConfigForm::WdsPage_OnDelete()
{
    QListViewItem *Item;

    Item = ListViewWds->currentItem();
    if(Item == NULL)
        return ;

    ListViewWds->takeItem(Item);
    WdsPushButtonAdd->setEnabled( TRUE );
}

void ApConfigForm::WdsPage_OnRemoveAll()
{
    ListViewWds->clear();
    WdsPushButtonAdd->setEnabled( TRUE );
}

bool ApConfigForm::WdsPage_WriteConfigToRegistryAndSetOid()
{
    QListViewItem   *viewItem;
    QString         slist;
    QString         str;
    QString         strWdsKey;
    RT_802_11_WDS   wds;
    ULONG           lBufLen;
    UCHAR           keyMaterial[40];
    char            tmp[255];
    int             j = 0;

    memset(&wds, 0x00, sizeof(RT_802_11_WDS));

    sprintf(tmp, "%d", RadioButtonWdsEnable->isChecked());
    writekeystring("Default", "WdsEnable", tmp, RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    wds.Num = ListViewWds->childCount();
    viewItem = ListViewWds->firstChild();
    for (ULONG i = 0; i < wds.Num; i++)
    {
        str=viewItem->text(0);
        j = 0;
        while ( ( j = str.find( '-', j ) ) != -1 )
            str.remove( j ,1 );

        AtoH((char *)str.data(), wds.MacAddress[i], 6);
        str.sprintf("%02X:%02X:%02X:%02X:%02X:%02X;", 
            wds.MacAddress[i][0], wds.MacAddress[i][1], wds.MacAddress[i][2], 
            wds.MacAddress[i][3], wds.MacAddress[i][4], wds.MacAddress[i][5]);
        slist.append(str);
        viewItem = viewItem->itemBelow();
    }

    writekeystring("Default", "WdsList", (char *)slist.data(), RT2500AP_SYSTEM_PATH, RT2500AP_SYSTEM_DATA);

    strWdsKey = m_pdlgAuthSecu->Get_WpaPsk_text();
    if (RadioButtonWdsEnable->isChecked()) //Enable
    {
        if (m_pdlgAuthSecu->Get_Wep_AuthenType() == Ndis802_11AuthModeWPAPSK)
        {
            wds.KeyLength = 32;
            PasswordHash((char *)strWdsKey.data(), (unsigned char *)RALINK, sizeof(RALINK), keyMaterial);

            memcpy(wds.KeyMaterial, keyMaterial, wds.KeyLength);
        }
        else
            wds.KeyLength = 0;
    }
    else
    {
        wds.Num = 0;
        wds.KeyLength = 0;
    }

    QMessageBox::information( this, "WDS",
            "<center><h1><font color=blue>WDS<font></h1></center>"
            "<p>To Enable the <i> <font color=blue> WDS</i>, you must to "
            "<i> <font color=red> re-load </i>the driver.</p>"
            );

    OidSetInformation(RT_OID_802_11_WDS, m_nSocket_Id, m_strDevice_Name, &wds, sizeof(RT_802_11_WDS));
}

void ApConfigForm::WdsPage_EnableControl(bool bEnable)
{
    ListViewWds->setEnabled( bEnable );
    LineEditWdsMacAddress->setEnabled( bEnable );

    if (ListViewWds->childCount() >= 3)
        WdsPushButtonAdd->setEnabled( FALSE );
    else
        WdsPushButtonAdd->setEnabled( bEnable );

    WdsPushButtonDelete->setEnabled( bEnable );
    WdsPushButtonRemoveAll->setEnabled( bEnable );
}

void ApConfigForm::GetCountryChannel()
{
    switch (CountryRegion)
    {
        case REGSTR_COUNTRYREGION_IC:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_IC, sizeof(Ra24Ghz_IC));
            m_n24GhzIndex = sizeof(Ra24Ghz_IC);
            break;
        case REGSTR_COUNTRYREGION_ETSI:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_ESTI, sizeof(Ra24Ghz_ESTI));
            m_n24GhzIndex = sizeof(Ra24Ghz_ESTI);
            break;
        case REGSTR_COUNTRYREGION_SPAIN:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_SPAIN, sizeof(Ra24Ghz_SPAIN));
            m_n24GhzIndex = sizeof(Ra24Ghz_SPAIN);
            break;
        case REGSTR_COUNTRYREGION_FRANCE:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_FRANCE, sizeof(Ra24Ghz_FRANCE));
            m_n24GhzIndex = sizeof(Ra24Ghz_FRANCE);
            break;
        case REGSTR_COUNTRYREGION_MKK:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_MKK, sizeof(Ra24Ghz_MKK));
            m_n24GhzIndex = sizeof(Ra24Ghz_MKK);
            break;
        case REGSTR_COUNTRYREGION_MKK1:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_MKK1, sizeof(Ra24Ghz_MKK1));
            m_n24GhzIndex = sizeof(Ra24Ghz_MKK1);
            break;
        case REGSTR_COUNTRYREGION_ISRAEL:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_ISRAEL, sizeof(Ra24Ghz_ISRAEL));
            m_n24GhzIndex = sizeof(Ra24Ghz_ISRAEL);
            break;
        case REGSTR_COUNTRYREGION_FCC:
        default:
            memcpy(m_chAryChan24Ghz, Ra24Ghz_FCC, sizeof(Ra24Ghz_FCC));
            m_n24GhzIndex = sizeof(Ra24Ghz_FCC);
            break;
    }

    memcpy(m_chAryChan52Ghz, Ra5Ghz_UNII, sizeof(Ra5Ghz_UNII));
    m_n52GhzIndex = sizeof(Ra5Ghz_UNII);
}

