/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology 5th Rd.
 * Science-based Industrial Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright 2002, Ralink Technology, Inc.
 *
 * All rights reserved. Ralink's source code is an unpublished work and the
 * use of a copyright notice does not imply otherwise. This source code
 * contains confidential trade secret material of Ralink Tech. Any attemp
 * or participation in deciphering, decoding, reverse engineering or in any
 * way altering the source code is stricitly prohibited, unless the prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************

    Module Name:
    rt_tool.cpp

    Abstract:
        genernal sub function for ioctl talk to driver.

    Revision History:
    Who            When          What
    --------    ----------      ----------------------------------------------
    Paul Wu     01-22-2003      created

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/wireless.h>

#include "rt_tool.h"
#include "sha1.h"

//
//  FUNCTION: BtoH(char *, char *, int)
//
//  PURPOSE:  Converts ascii byte to numeric
//
//  PARAMETERS:
//    ch - ascii byte to convert
//
//  RETURNS:
//    associated numeric value
//
//  COMMENTS:
//
//    Will convert any hex ascii digit to its numeric counterpart.
//    Puts in 0xff if not a valid hex digit.
//

static UCHAR BtoH(char ch)
{
    if (ch >= '0' && ch <= '9') return (ch - '0');        // Handle numerals
    if (ch >= 'A' && ch <= 'F') return (ch - 'A' + 0xA);  // Handle capitol hex digits
    if (ch >= 'a' && ch <= 'f') return (ch - 'a' + 0xA);  // Handle small hex digits
    return(255);
}

//
//  FUNCTION: AtoH(char *, UCHAR *, int)
//
//  PURPOSE:  Converts ascii string to network order hex
//
//  PARAMETERS:
//    src    - pointer to input ascii string
//    dest   - pointer to output hex
//    destlen - size of dest
//
//  COMMENTS:
//
//    2 ascii bytes make a hex byte so must put 1st ascii byte of pair
//    into upper nibble and 2nd ascii byte of pair into lower nibble.
//

void AtoH(char * src, UCHAR * dest, int destlen)
{
    char * srcptr;

    srcptr = src;
    PUCHAR destTemp = dest; 

    while(destlen--)
    {
        *destTemp = BtoH(*srcptr++) << 4;    // Put 1st ascii byte in upper nibble.
        *destTemp += BtoH(*srcptr++);      // Add 2nd ascii byte to above.
        destTemp++;
    }
}

/*------------------------------------------------------------------*/
/*
 * Open a socket.
 * Depending on the protocol present, open the right socket. The socket
 * will allow us to talk to the driver.
 */
int Open_Socket(void)
{
    static const int families[] = {
        AF_INET, AF_IPX, AF_AX25, AF_APPLETALK
    };
    unsigned int    i;
    int        sock;

    /*
    * Now pick any (exisiting) useful socket family for generic queries
    * Note : don't open all the socket, only returns when one matches,
    * all protocols might not be valid.
    * Workaround by Jim Kaba <jkaba@sarnoff.com>
    * Note : in 99% of the case, we will just open the inet_sock.
    * The remaining 1% case are not fully correct...
    */

    /* Try all families we support */
    for(i = 0; i < sizeof(families)/sizeof(int); ++i)
    {
        /* Try to open the socket, if success returns it */
        sock = socket(families[i], SOCK_DGRAM, 0);
        if(sock >= 0)
            return sock;
    }

    return -1;
}

int OidQueryInformation(USHORT OidQueryCode, int socket_id, char *DeviceName, void *ptr, ULONG PtrLength)
{
    struct iwreq wrq;

    strcpy(wrq.ifr_name, DeviceName);
    wrq.u.data.length = PtrLength;
    wrq.u.data.pointer = (caddr_t) ptr;
    wrq.u.data.flags = OidQueryCode;

    return (ioctl(socket_id, RT_PRIV_IOCTL, &wrq));
}

int OidSetInformation(USHORT OidQueryCode, int socket_id, char *DeviceName, void *ptr, ULONG PtrLength)
{
    struct iwreq wrq;

    strcpy(wrq.ifr_name, DeviceName);
    wrq.u.data.length = PtrLength;
    wrq.u.data.pointer = (caddr_t) ptr;
    wrq.u.data.flags = OidQueryCode | OID_GET_SET_TOGGLE;

    return (ioctl(socket_id, RT_PRIV_IOCTL, &wrq));
}
// most likely OID_802_11_RSSI ranged from -20 to -100 dbm
//  -40 dBm (100%) ------- -80 dBm (20%) ------- -84 dBm (0%)
//
unsigned int  ConvertRssiToSignalQuality(int RSSI)
{
    unsigned int signal_quality;
    if (RSSI >= -40 )
        signal_quality = 100;
    else if (RSSI >= -80)    // between -40 ~ -80dbm
        signal_quality = 20 + (RSSI + 80) * 2;   
    else if (RSSI >= -84 )   // between -80 ~ -84dbm
        signal_quality = (RSSI + 84) * 5;   
    else    // < -84 dbm
        signal_quality = 0;

    return signal_quality;
}

void unmktime(unsigned long time, long offset, int *yearp, int *monp, int *dayp, int *hourp, int *minp, int *secp)
{
    /* How many days come before each month (0-12).  */
    static const unsigned short int __mon_yday[2][13] =
    {
        /* Normal years.  */
        { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 },
        /* Leap years.  */
        { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }
    };

    long int days, rem, y, wday, yday;
    const unsigned short int *ip;

    #define SECS_PER_MINUTE (60)
    #define SECS_PER_HOUR   (60 * 60)
    #define SECS_PER_DAY    (SECS_PER_HOUR * 24)

    days = time / SECS_PER_DAY;
    rem = time % SECS_PER_DAY;
    rem += offset;

    while (rem < 0) 
    {
        rem += SECS_PER_DAY;
        --days;
    }
    while (rem >= SECS_PER_DAY)
    {
        rem -= SECS_PER_DAY;
        ++days;
    }

    *hourp = rem / SECS_PER_HOUR;
    rem %= SECS_PER_HOUR;
    *minp = rem / SECS_PER_MINUTE;
    *secp = rem % SECS_PER_MINUTE;    /* January 1, 1970 was a Thursday. */
    wday = (4 + days) % 7; /* Day in the week. Not currently used */
    if (wday < 0) 
        wday += 7;
    y = 1970;

    #define DIV(a, b)                   ((a) / (b) - ((a) % (b) < 0))
    #define LEAPS_THRU_END_OF(y)        (DIV (y, 4) - DIV (y, 100) + DIV (y, 400))
    #define __isleap(year)              ((year) % 4 == 0 && ((year) % 100 != 0 || (year) % 400 == 0))

    while (days < 0 || days >= (__isleap (y) ? 366 : 365))
    {
        /* Guess a corrected year, assuming 365 days per year.  */
        long int yg = y + days / 365 - (days % 365 < 0);        /* Adjust DAYS and Y to match the guessed year.  */
        days -= ((yg - y) * 365 + LEAPS_THRU_END_OF (yg - 1) - LEAPS_THRU_END_OF (y - 1));
        y = yg;
    }
    *yearp = y;
    yday = days; /* day in the year.  Not currently used. */
    ip = __mon_yday[__isleap(y)];

    for (y = 11; days < (long int) ip[y]; --y)
        continue;
    days -= ip[y];
    *monp = y + 1;    // modify by rory
    *dayp = days + 1; /* day in the month */

    return;
}

static void hmac_sha1(unsigned char *text, int text_len, unsigned char *key, int key_len, unsigned char *digest) 
{ 
    A_SHA_CTX context; 
    unsigned char k_ipad[65]; /* inner padding - key XORd with ipad */ 
    unsigned char k_opad[65]; /* outer padding - key XORd with opad */ 
    int i; 

    /* if key is longer than 64 bytes reset it to key=SHA1(key) */ 
    if (key_len > 64) 
    { 
        A_SHA_CTX tctx; 

        A_SHAInit(&tctx); 
        A_SHAUpdate(&tctx, key, key_len); 
        A_SHAFinal(&tctx, key); 

        key_len = 20; 
    } 

    /* 
    * the HMAC_SHA1 transform looks like: 
    * 
    * SHA1(K XOR opad, SHA1(K XOR ipad, text)) 
    * 
    * where K is an n byte key 
    * ipad is the byte 0x36 repeated 64 times 
    * opad is the byte 0x5c repeated 64 times 
    * and text is the data being protected 
    */ 

    /* start out by storing key in pads */ 
    memset(k_ipad, 0, sizeof k_ipad); 
    memset(k_opad, 0, sizeof k_opad); 
    memcpy(k_ipad, key, key_len); 
    memcpy(k_opad, key, key_len); 

    /* XOR key with ipad and opad values */ 
    for (i = 0; i < 64; i++) 
    { 
        k_ipad[i] ^= 0x36; 
        k_opad[i] ^= 0x5c; 
    } 

    /* perform inner SHA1*/ 
    A_SHAInit(&context); /* init context for 1st pass */ 
    A_SHAUpdate(&context, k_ipad, 64); /* start with inner pad */ 
    A_SHAUpdate(&context, text, text_len); /* then text of datagram */ 
    A_SHAFinal(&context, digest); /* finish up 1st pass */ 

    /* perform outer SHA1 */ 
    A_SHAInit(&context); /* init context for 2nd pass */ 
    A_SHAUpdate(&context, k_opad, 64); /* start with outer pad */ 
    A_SHAUpdate(&context, digest, 20); /* then results of 1st hash */ 
    A_SHAFinal(&context, digest); /* finish up 2nd pass */ 
} 

/*
* F(P, S, c, i) = U1 xor U2 xor ... Uc 
* U1 = PRF(P, S || Int(i)) 
* U2 = PRF(P, U1) 
* Uc = PRF(P, Uc-1) 
*/ 

static void F(char *password, unsigned char *ssid, int ssidlength, int iterations, int count, unsigned char *output) 
{ 
    unsigned char digest[36], digest1[A_SHA_DIGEST_LEN]; 
    int i, j; 

    /* U1 = PRF(P, S || int(i)) */ 
    memcpy(digest, ssid, ssidlength); 
    digest[ssidlength] = (unsigned char)((count>>24) & 0xff); 
    digest[ssidlength+1] = (unsigned char)((count>>16) & 0xff); 
    digest[ssidlength+2] = (unsigned char)((count>>8) & 0xff); 
    digest[ssidlength+3] = (unsigned char)(count & 0xff); 
    hmac_sha1(digest, ssidlength+4, (unsigned char*) password, (int) strlen(password), digest1); // for WPA update

    /* output = U1 */ 
    memcpy(output, digest1, A_SHA_DIGEST_LEN); 

    for (i = 1; i < iterations; i++) 
    { 
        /* Un = PRF(P, Un-1) */ 
        hmac_sha1(digest1, A_SHA_DIGEST_LEN, (unsigned char*) password, (int) strlen(password), digest); // for WPA update
        memcpy(digest1, digest, A_SHA_DIGEST_LEN); 

        /* output = output xor Un */ 
        for (j = 0; j < A_SHA_DIGEST_LEN; j++) 
        { 
            output[j] ^= digest[j]; 
        } 
    } 
} 
/* 
* password - ascii string up to 63 characters in length 
* ssid - octet string up to 32 octets 
* ssidlength - length of ssid in octets 
* output must be 40 octets in length and outputs 256 bits of key 
*/ 
int PasswordHash(char *password, unsigned char *ssid, int ssidlength, unsigned char *output) 
{ 
    if ((strlen(password) > 63) || (ssidlength > 32)) 
        return 0; 

    F(password, ssid, ssidlength, 4096, 1, output); 
    F(password, ssid, ssidlength, 4096, 2, &output[A_SHA_DIGEST_LEN]); 
    return 1; 
}

